//*******************************************************************
//
// Company:		Aculab, Inc.
//
// Author:		Jeff Feddon
//
// FileName:	AculabClass.cpp 
//
// Date:		04/09/2007
//
// Description:	Wrapper class for Aculab API
//
//******************************************************************

#include "AculabClass.h"

int				AculabClass::initialized=0;
int				AculabClass::iExit=0;
ACU_UINT		AculabClass::unCardId=0;
ACU_PORT_ID		AculabClass::cSipPortId=0;
int				AculabClass::modules=0;
ACU_UINT		AculabClass::unPorts=0;
ACU_CHAR		AculabClass::card_ip_address[ACU_MAX_IP_ADDRESS];
PORT_DATA*		AculabClass::pPortData;
MODULE_DATA*	AculabClass::pModuleData;

CRITICAL_SECTION	LOG_CS;

AculabClass::AculabClass()
{
	//LogPrint("INIT: %d", initialized);
	class_type = ACU_CLASS;
}

AculabClass::~AculabClass()
{

}

void AculabClass::Init()
{
	ACU_UINT				i,x;
	ACU_INT					m;
	ACU_ERR					rc;
	ACU_UINT				unCards;
	
	ACU_SNAPSHOT_PARMS		rSnapshotParms; 
	ACU_OPEN_CARD_PARMS		rOpenParms;
	ACU_CARD_INFO_PARMS		rCardInfo;

	ACU_OPEN_PROSODY_PARMS	rOpenSpeech;
	SM_CARD_INFO_PARMS		sCardInfo;
	SM_OPEN_MODULE_PARMS	sOpenModule;

	ACU_PORT_ID				cSipPortId;

#if TINGLOGTOFILE
	TiNGtrace = 3;
	TiNG_showtrace = &TingLogPrint;
#endif
	
	InitializeCriticalSection(&LOG_CS);

	/* Interrogate System */
	INIT_ACU_STRUCT(&rSnapshotParms);
	rc = acu_get_system_snapshot(&rSnapshotParms);
	if(rc) ErrorPrint("main", "acu_get_system_snapshot", rc);
	
	printf("CARDS = %d\n------------\n", unCards = rSnapshotParms.count);

	for(i=0;i<rSnapshotParms.count;i++) 
		LogPrint("SN%d = %s", i+1, rSnapshotParms.serial_no[i]);

	for(x=0;x<1;x++) 
	{
		/* Open Card */
		INIT_ACU_STRUCT(&rOpenParms);
		strcpy(rOpenParms.serial_no, rSnapshotParms.serial_no[x]);
		rc = acu_open_card(&rOpenParms);
		if(rc) ErrorPrint("main", "acu_open_card", rc);

		LogPrint("CARD %d Opened as %d", x+1, unCardId = rOpenParms.card_id);

		/* Get Card Info */
		INIT_ACU_STRUCT(&rCardInfo);
		rCardInfo.card_id = unCardId;
		rc = acu_get_card_info(&rCardInfo);
		if(rc) ErrorPrint("main", "acu_get_card_info", rc);

		strcpy(card_ip_address, rCardInfo.ip_address);

		/*
        PIP_ADAPTER_INFO pAdapterInfo = NULL;
        ULONG OutBufferLength = 0;
        for (i = 0; i < 2; i++) 
        {
			rc = GetAdaptersInfo(pAdapterInfo, &OutBufferLength);
            if(rc != ERROR_BUFFER_OVERFLOW) break;
            if(pAdapterInfo != NULL) free(pAdapterInfo);

            pAdapterInfo = (PIP_ADAPTER_INFO) calloc(1, OutBufferLength);
		}
		strcpy(card_ip_address, pAdapterInfo->IpAddressList.IpAddress.String);
        free(pAdapterInfo);
		*/
		
		/* Open Speech Driver Interface */
		INIT_ACU_STRUCT(&rOpenSpeech);
		rOpenSpeech.card_id = unCardId;
		rc = acu_open_prosody(&rOpenSpeech);
		if(rc) ErrorPrint("main", "acu_open_prosody", rc);

		LogPrint("Open SPEECH interface on %d", unCardId);

		/* Get Speech Module Count */
		INIT_ACU_SM_STRUCT(&sCardInfo);
		sCardInfo.card = unCardId;
		rc = sm_get_card_info(&sCardInfo);
		if(rc) ErrorPrint("main", "sm_get_card_info", rc);

		/* Allocate Module Data Array */
		pModuleData = (MODULE_DATA*) calloc(sCardInfo.module_count, sizeof(MODULE_DATA));
		memset(pModuleData,0,sCardInfo.module_count*sizeof(MODULE_DATA));

		LogPrint("Modules on Card %d = %d", x+1, modules=sCardInfo.module_count);

		/* Open Prosody Modules */
		for(m=0;m<sCardInfo.module_count;m++)
		{
			INIT_ACU_SM_STRUCT(&sOpenModule);
			sOpenModule.module_ix = m;
			sOpenModule.card_id = unCardId;
			rc = sm_open_module(&sOpenModule);
			if(rc) ErrorPrint("main", "sm_open_module", rc);
			
			LogPrint("MODULE: %d is OPENED", m);

			pModuleData[m].module = m;
			pModuleData[m].module_id = sOpenModule.module_id;
		}
	
		unPorts = 1;

		LogPrint("Card %s has %d ports", rSnapshotParms.serial_no[x], unPorts);
					
		/* Allocate Port Data Array */
		pPortData = (PORT_DATA*) calloc(unPorts, sizeof(PORT_DATA));
		memset(pPortData,0,unPorts*sizeof(PORT_DATA));

		i=0;

		rc = sip_open_port(&cSipPortId);
		if(rc) ErrorPrint("main", "sip_open_port", rc);

		pPortData[i].done_event = CreateEvent(NULL,FALSE,FALSE,NULL);
		pPortData[i].port = i;
		pPortData[i].port_id = cSipPortId;
		InitializeCriticalSection(&pPortData[i].IN_CS );
		InitializeCriticalSection(&pPortData[i].OUT_CS );

		/* Allocate Call Data Array */
		pPortData[i].pBChanDataIN = (CALL_DATA*) calloc(NUMCALLSPERPORT, sizeof(CALL_DATA));
		memset(pPortData[i].pBChanDataIN,0,NUMCALLSPERPORT*sizeof(CALL_DATA));		

		/* Allocate Call Data Array */
		pPortData[i].pBChanDataOUT = (CALL_DATA*) calloc(NUMCALLSPERPORT, sizeof(CALL_DATA));
		memset(pPortData[i].pBChanDataOUT,0,NUMCALLSPERPORT*sizeof(CALL_DATA));	
	}

	initialized = 1;
}

void AculabClass::Cleanup()
{
	int						m,rc;
	unsigned int			i=0;
	SM_CLOSE_MODULE_PARMS	sCloseModule;
	ACU_CLOSE_PROSODY_PARMS	rCloseSpeech;
	ACU_CLOSE_CARD_PARMS	rCloseParms;

	iExit=1;

	Sleep(5000);

	for(m=0;m<modules;m++)
	{
		INIT_ACU_SM_STRUCT(&sCloseModule);
		sCloseModule.module_id = pModuleData[m].module_id;
		rc = sm_close_module(&sCloseModule);
		if(rc) ErrorPrint("main", "sm_close_module", rc);
		LogPrint("MODULE: %d is CLOSED", pModuleData[m].module);
	}
	
	INIT_ACU_STRUCT(&rCloseSpeech);
	rCloseSpeech.card_id = unCardId;
	rc = acu_close_prosody(&rCloseSpeech);
	if(rc) ErrorPrint("main", "acu_close_prosody", rc);
		
	LogPrint("Close SPEECH interface on %d", unCardId);

	INIT_ACU_STRUCT(&rCloseParms);
	rCloseParms.card_id = unCardId;
	rc = acu_close_card(&rCloseParms);
	if(rc) ErrorPrint("main", "acu_close_card", rc);

	/* Free up Allocated Memory */
	for(i=0;i<unPorts; i++) 
	{
		DeleteCriticalSection(&pPortData[i].IN_CS );
		free( pPortData[i].pBChanDataIN );
		DeleteCriticalSection(&pPortData[i].OUT_CS );
		free( pPortData[i].pBChanDataOUT );
	}

	DeleteCriticalSection(&LOG_CS);

	free( pPortData );
	free( pModuleData );
	
	printf("\n==================");  
	printf("\nACULAB CARD CLOSED");  
	printf("\n==================\n");  
}

void AculabClass::ErrorPrint(char *InFunction, char *Call, int rc)
{
	LogPrint("In %s, %s failed and return code: %d", InFunction, Call, rc);
}

void AculabClass::LogPrint(char* format, ...)
{
	struct _timeb			timebuffer;
	char					*timeline;
	va_list					arglist;
	
#if LOGTOFILE
	static FILE*			fp;
	static int				EnterOnce = 0;

	if(!EnterOnce)
    {
		fp = fopen("output.log", "w+");
		EnterOnce = 1;
	}	
#endif

	EnterCriticalSection(&LOG_CS);
	_ftime( &timebuffer );
	timeline = ctime( & ( timebuffer.time ) );
	printf( "[%.8s.%hu]  ", timeline+11, timebuffer.millitm );

	va_start(arglist, format);
	vprintf(format, arglist);
	printf("\n");

#if LOGTOFILE
	fprintf(fp, "[%.8s.%hu]  ", timeline+11, timebuffer.millitm );
	vfprintf(fp, format, arglist);
	fprintf(fp, "\n");
	fflush(fp);
#endif

	va_end(arglist);
	fflush(stdout);
	LeaveCriticalSection(&LOG_CS);
}

int AculabClass::TingLogPrint(const char* format, va_list arglist)
{
	static int					EnterOnce = 0;
    static FILE*				fp;
 
	if(!EnterOnce)
    {
        fp = fopen("ting.log", "w+");
		EnterOnce = 1;
	}
    
	//EnterCriticalSection(&LOG_CS);
    vfprintf(fp, format, arglist);
    fflush(fp);
    //LeaveCriticalSection(&LOG_CS);

	return 0;
}
