//*******************************************************************
//
// Company:		Aculab, Inc.
//
// Author:		Jeff Feddon
//
// FileName:	AculabDriver.cpp 
//
// Date:		04/13/2007
//
// Description:	Sample Driver for Aculab Wrapper Classes
//
//******************************************************************

//Aculab header files:
#include <stdio.h>
#include "AculabClass.h"
#include "IncomingClass.h"
#include "OutgoingClass.h"
#include "DTMFClass.h"

//Sangoma header files:
#include "sangoma_interface_manager.h"
#define DBG_SANG_MAIN	if(1)printf
#define ERR_SANG_MAIN	if(1)printf
#define WRN_SANG_MAIN	if(1)printf

#define DBG_ACU_TS		if(1)printf

#if USE_SANGOMA
sangoma_interface_manager *s_if_manager = NULL;
sangoma_callback_functions_t s_callback_functions;

//prototypes
static int sangoma_accepted_incoming_sip_call(void *aculab_object, int timeslot);
static int sangoma_outgoing_sip_call(void *sangoma_object);
static int sangoma_on_hook(void *sangoma_object);

#endif

#if 1

#include "sangoma_interface.h"

void IncomingDetectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot, char* DestNum);
void IncomingConnectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot);
void IncomingIdleCallback(ACU_CALL_HANDLE CallHandle, int Timeslot);

void OutgoingConnectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot);
void OutgoingIdleCallback(ACU_CALL_HANDLE CallHandle, int Timeslot);

void DTMFDetectedCallback(char*,class sangoma_interface*);

IncomingClass	*pI;
OutgoingClass	*pO;
//DTMFClass		*pD;

class AculabTimeslots
{
	int TimeslotAvailable[NUMCALLSPERPORT];

	CRITICAL_SECTION	AcuTsCriticalSection;

public:

	AculabTimeslots()
	{
		DBG_ACU_TS("AculabTimeslots()\n");
		InitializeCriticalSection(&AcuTsCriticalSection);
		memset( TimeslotAvailable, 0, NUMCALLSPERPORT * sizeof(int) );
	}

	~AculabTimeslots()
	{
		DBG_ACU_TS("~AculabTimeslots()\n");
	}

	//Returns: -1 - if no free timeslots fount
	//			or the free timeslot number
	int find_free_slot()
	{
		int rc = -1;
		EnterCriticalSection(&AcuTsCriticalSection); 

		for(int i = 0; i < NUMCALLSPERPORT; i++)
		{	// Timeslot is available
			if( TimeslotAvailable[ i ] == 0)
			{
				rc = i;
				break;
			}
		}

		LeaveCriticalSection(&AcuTsCriticalSection);
		DBG_ACU_TS("find_free_slot() : returning Acu timeslot: %d\n", rc);
		return rc;
	}

	int is_valid_timeslot(int ts)
	{
		if(ts < 0 || ts >= NUMCALLSPERPORT){
			//timeslot is invalid (out of range)
			return 1;
		}else{
			return 0;
		}
	}

	//Argumet(s): timeslot
	//Returns	: -1 - if requested timeslot is busy
	//			  or out of valid range.
	//			  0  - ok
	int set_slot_to_busy(int ts)
	{
		if(is_valid_timeslot(ts)){
			return -1;
		}

		EnterCriticalSection(&AcuTsCriticalSection); 
		TimeslotAvailable[ ts ] = 1;
		LeaveCriticalSection(&AcuTsCriticalSection);
		return 0;
	}

	//Argumet(s): timeslot
	//Returns	: -1 - if requested timeslot is out of valid range
	//			  0  - ok
	int set_slot_to_free(int ts)
	{
		if(is_valid_timeslot(ts)){
			return -1;
		}

		EnterCriticalSection(&AcuTsCriticalSection); 
		TimeslotAvailable[ ts ] = 0;
		LeaveCriticalSection(&AcuTsCriticalSection);
		return 0;
	}
};

AculabTimeslots Acu_ts;

int main(int argc, char* argv[])
{
	int i;
	char ch;

#if USE_SANGOMA
	///////////////////////////////////////////////////////////////////////////////////
	//initialize callback function pointers
	s_callback_functions.sangoma_accepted_incoming_sip_call = sangoma_accepted_incoming_sip_call;
	s_callback_functions.sangoma_outgoing_sip_call = sangoma_outgoing_sip_call;
	s_callback_functions.sangoma_on_hook = sangoma_on_hook;

	s_if_manager = new sangoma_interface_manager();
	if(s_if_manager->build_list_of_interfaces(&s_callback_functions)){
		delete s_if_manager;
		return 1;
	}else{
		printf("Successfuly initialized Sangoma Interfaces.\n");
	}
	///////////////////////////////////////////////////////////////////////////////////
#endif
	
	AculabClass A;
	A.Init();

	IncomingClass I;
	I.SetIncomingDetectedCallback(&IncomingDetectedCallback);
	I.SetIncomingConnectedCallback(&IncomingConnectedCallback);
	I.SetIncomingIdleCallback(&IncomingIdleCallback);
	pI = &I;

	// Open maximum of Incoming Calls allowed by Aculab license.
	for(i=0; i < NUMCALLSPERPORT; i++){
		I.OpenIncoming(i);
	}
	
	//Sleep(1000);

	OutgoingClass O;
	O.SetOutgoingConnectedCallback(&OutgoingConnectedCallback);
	O.SetOutgoingIdleCallback(&OutgoingIdleCallback);
	pO = &O;
/*
	DTMFClass D;
	D.SetDTMFDetectedCallback(&DTMFDetectedCallback);
	pD = &D;
*/	
	// #2 Assuming Analog Phone has gone OFF HOOK Start DTMF Detection
	// Param 1: Sangoma Object
	// Param 2: Timeout...after user finished dialing...wait for "timeout" 
	//          before terminating DTMF Detection
	                                                                
	//D.DoDTMFDetection(&S[0], 1000);

	//D.DoDTMFGeneration("123456789", &S[0]);
	
#if 0
	int j;
	for(i=0; ; i++)
	{
		for(j=0; j < 10; j++) 
		{
			O.OpenOutgoing(j,"127.0.0.1","127.0.0.1", &S[i]);
		}
	
		Sleep(1000);

		for(j=0; j < 10; j++)
		{
			O.DisconnectCall(j);
		}

		Sleep(1000);
	}
#endif

	while((ch=_getch()) != 'q')
		Sleep(100);

#if USE_SANGOMA
	//1-st disconnect from Sangoma API. Important!!
	delete s_if_manager;

	printf("===================\n");  
	printf("SANGOMA API CLOSED\n");  
	printf("===================\n");  
#endif

	A.Cleanup();
	return 0;
}

//*******************************************************************
// Aculab callback functions:
//*******************************************************************

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//Incoming
///////////////////////////////////////////////////////////////////////////////////////////////////////////

void IncomingDetectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot, char* DestNum)
{
	sangoma_interface * sang_if;

	// #5 Incoming Call with DestNum Digits in the SIP INVITE
	pI->LogPrint("INCOMING **DETECTED** CALLBACK ON HANDLE %x : TS %d : Dest: %s", 
											CallHandle, Timeslot,DestNum);

	if((sang_if = s_if_manager->find_free_interface()) == NULL){
		DBG_SANG_MAIN("No free Sangoma interfaces!!\n");
		//This the same as "No Answer" condition from the Analog Side
		pI->DisconnectCall(Timeslot);
		return;
	}

	sang_if->incoming_call(pI, Timeslot, "CID from SIP");

	// Set the Sangoma Object 
	pI->SetSangomaObject(sang_if, Timeslot);

	// #6 Generate DTMF to Analog side
	//pD->DoDTMFGeneration(DestNum, sang_if);
}

void IncomingConnectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot)
{
	pI->LogPrint("INCOMING **CONNECTED** ON HANDLE %x : TS %d", CallHandle, Timeslot);

	//pD->DoDTMFDetection((sangoma_interface *)pI->GetSangomaObject(Timeslot), 1000);

	DBG_SANG_MAIN("IncomingConnectedCallback() - end\n");
}

void IncomingIdleCallback(ACU_CALL_HANDLE CallHandle, int Timeslot)
{
	sangoma_interface * sang_if;

	pI->LogPrint("INCOMING **IDLE** CALLBACK ON HANDLE %x : TS %d", CallHandle, Timeslot);
	
	Acu_ts.set_slot_to_free(Timeslot);

	// Call finished...Open back up for incoming
	pI->OpenIncoming(Timeslot);

	sang_if = (sangoma_interface *)pI->GetSangomaObject(Timeslot);

	if(sang_if != NULL){
		sang_if->incoming_call_idle();
	}else{
		WRN_SANG_MAIN("WARNING: IncomingIdleCallback(): No Sangoma interface on Call End! (Timeslot: %d)\n", 
			Timeslot);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//Outgoing
///////////////////////////////////////////////////////////////////////////////////////////////////////////
void OutgoingConnectedCallback(ACU_CALL_HANDLE CallHandle, int Timeslot)
{
	sangoma_interface * sang_if;

	pO->LogPrint("OUTGOING **CONNECTED** CALLBACK ON HANDLE %x : TS %d", CallHandle, Timeslot);

	Acu_ts.set_slot_to_busy(Timeslot);

	sang_if = (sangoma_interface *)pO->GetSangomaObject(Timeslot);

	if(sang_if != NULL){
		sang_if->outgoing_call_connected();
	}else{
		WRN_SANG_MAIN("WARNING: (1) No Sangoma interface on Call Start! (Timeslot: %d)\n", Timeslot);
		return;
	}
}

void OutgoingIdleCallback(ACU_CALL_HANDLE CallHandle, int Timeslot)
{
	sangoma_interface * sang_if;

	pO->LogPrint("OUTGOING **IDLE** CALLBACK ON HANDLE %x : TS %d", CallHandle, Timeslot);

	// Call finished...Open back up for incoming
	Acu_ts.set_slot_to_free(Timeslot);

	// Call finished...Open back up for incoming
	pI->OpenIncoming(Timeslot);

	sang_if = (sangoma_interface *)pO->GetSangomaObject(Timeslot);

	if(sang_if != NULL){
		sang_if->outgoing_call_idle();
	}else{
		WRN_SANG_MAIN("WARNING: OutgoingIdleCallback(): No Sangoma interface on Call End! (Timeslot: %d)\n", 
			Timeslot);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//DTMF
///////////////////////////////////////////////////////////////////////////////////////////////////////////

void DTMFDetectedCallback(char* DestNum, class sangoma_interface* so)
{
//	char destination[30];

	// #3 DTMF Detected -- Caller entered DestNum

//	pD->LogPrint("DTMF **DETECTED** CALLBACK ... Digits: %s", DestNum);

//	if(DestNum != NULL) strcpy(destination, DestNum);

//	strcat(destination, "@127.0.0.1");

	// #4 Make Outgoing Call using DestNum@IPAddress
	// The Incoming call will receive this ( see #5 above )

//	pO->OpenOutgoing(0,"127.0.0.1",destination, so);
}

//*******************************************************************
// End of Aculab callback functions.
//*******************************************************************

//*******************************************************************
// Sangoma callback functions:
//*******************************************************************
//Analog side accepted incoming call (for example phone went Off Hook)
static int sangoma_accepted_incoming_sip_call(void *aculab_object, int timeslot)
{
	sangoma_interface * sang_if;

	DBG_SANG_MAIN("sangoma_accepted_incoming_sip_call(): timeslot: %d\n", timeslot);
	
	IncomingClass	*pInc = (IncomingClass*)aculab_object;

	sang_if = (sangoma_interface *)pI->GetSangomaObject(timeslot);

	if(sang_if != NULL){
		sang_if->set_if_state(IF_BUSY);
	}else{
		WRN_SANG_MAIN("WARNING: No Sangoma interface on Call Start! (Timeslot: %d)\n", timeslot);
		return 1;
	}

	Acu_ts.set_slot_to_busy(timeslot);

	// Analog Side answered
	pInc->Send200OK(timeslot);

	return 0;
}

//Analog side wants to make outgoing call over SIP.
static int sangoma_outgoing_sip_call(void *sangoma_object)
{
	sangoma_interface * sang_if = (sangoma_interface *)sangoma_object;
	int timeslot;
	
	DBG_SANG_MAIN("sangoma_outgoing_sip_call()\n");

	timeslot = Acu_ts.find_free_slot();
	if(timeslot < 0){
		//SIP side has no free timeslots
		//Play busy tone to Analog phone
		sang_if->busy_condition();
		return 1;
	}

	DBG_SANG_MAIN("%s: timeslot: %d, Src IP: %s, Dest IP: %s\n",
		sang_if->device_name, timeslot, 
		LOCAL_CONNECTION_ADDRESS, sang_if->get_outgoing_ip_address());

	//store Aculab info and do some additional work
	sang_if->outgoing_call(pO, timeslot);

	pO->SetSangomaObject(sang_if, timeslot);

	pO->OpenOutgoing(	timeslot,
						LOCAL_CONNECTION_ADDRESS,
						sang_if->get_outgoing_ip_address(),
						sang_if);

	DBG_SANG_MAIN("%s: sangoma_outgoing_sip_call() - end\n", sang_if->device_name);

	return 0;
}

//Analog side went On Hook
static int sangoma_on_hook(void *sangoma_object)
{
	sangoma_interface * sang_if = (sangoma_interface *)sangoma_object;
	int timeslot;
	AculabClass *ac;
	
	DBG_SANG_MAIN("sangoma_on_hook()\n");

	ac = (AculabClass *)sang_if->get_aculab_obj();
	if(ac != NULL){
		timeslot = sang_if->get_aculab_timeslot();
		switch(ac->get_class_type())
		{
		case ACU_INCOMING:
			if(sang_if->get_if_state("sangoma_on_hook(1)") == IF_BUSY){
				((IncomingClass*)ac)->DisconnectCall(timeslot);
			}else{
				//??? what to do if call was not connected??
			}
			break;
		case ACU_OUTGOING:
			if(sang_if->get_if_state("sangoma_on_hook(2)") == IF_BUSY){
				((OutgoingClass*)ac)->DisconnectCall(timeslot);
			}else{
				//??? may be some CloseOutgoing() should be called if call was NOT answered??
			}
			break;
		default:
			ERR_SANG_MAIN("ERROR: sangoma_on_hook(): unknown ACU Class Type (%d))\n", 
				ac->get_class_type());
		}

	}else{
		sang_if->set_if_state(IF_FREE);
		sang_if->reset_interface_state();
	}

	return 0;
}

//*******************************************************************
// End of Sangoma callback functions.
//*******************************************************************

#else

int main(int argc, char* argv[])
{
	s_if_manager = new sangoma_interface_manager();
	if(s_if_manager->build_list_of_interfaces()){
		return 1;
	}

	DBG_SANG_MAIN("======================\n");

	delete s_if_manager;
	
	return 0;
}
#endif
