//*******************************************************************
//
// Company:		Aculab, Inc.
//
// Author:		Jeff Feddon
//
// FileName:	AculabClass.cpp 
//
// Date:		04/18/2007
//
// Description:	Wrapper class for DTMF Detection and Generation API
//
//******************************************************************

#include "AculabClass.h"
#include "DTMFClass.h"

int					DTMFClass::init_once=0;
DTMF_DATA*			DTMFClass::pDTMFData;
	
typedef struct _Thread_Data_
{
	DTMFClass*		DCP;
	int				ts;
} THREAD_DATA;

DWORD WINAPI ProsodyDTMFHandlerThread(LPVOID pParam)
{
	THREAD_DATA *TD = (THREAD_DATA*) pParam;
	DTMFClass	*pPtr = (DTMFClass*) TD->DCP;
	int i = TD->ts;
	free(TD);

	DTMF_DATA* DTMFData = ( DTMF_DATA* ) &pPtr->pDTMFData[i];
	
	int									rc,x;
	int									iDone;
	int									Count = 0;
	DWORD								dw;
	char								buffer[kSMMaxReplayDataBufferSize];		

	SM_CHANNEL_ALLOC_PLACED_PARMS		sChannelParms_1;
	SM_CHANNEL_ALLOC_PLACED_PARMS		sChannelParms_2;

	SM_CHANNEL_DATAFEED_PARMS			chan_datafeed;
	SM_CHANNEL_DATAFEED_CONNECT_PARMS	feed_connect;

	SM_LISTEN_FOR_PARMS					sListenParms;
	SM_RECOGNISED_PARMS					sRecogParms;
	SM_PLAY_DIGITS_PARMS				sPlayDTMF;
	SM_PLAY_DIGITS_STATUS_PARMS			sStatusParms;

	SM_RECORD_PARMS						sRecordParms;
	SM_RECORD_ABORT_PARMS				sRecordAbortParms;
	SM_RECORD_STATUS_PARMS				sRecordStatus;

	SM_REPLAY_STATUS_PARMS				sReplayStatusParms;
	SM_REPLAY_PARMS						sReplayParms;
	SM_REPLAY_ABORT_PARMS				sPlayAbortParms;
	SM_TS_DATA_PARMS					sm_ts_data_parms;

	SM_CHANNEL_SET_EVENT_PARMS			sEventParmsRecord;
	SM_CHANNEL_SET_EVENT_PARMS			sEventParmsPlay;
	SM_CHANNEL_SET_EVENT_PARMS			sEventParmsDTMF;
	HANDLE								hEvents[2];
	
	struct _timeb						timebuffer;
	long								mstime1, mstime2;
	DWORD								waittimeout;

#if USE_SANGOMA
	static int							sangoma_dbg_counter = 0;
#else
	#if TEST_SANGOMA_STUBS

	#else
		char							szBuffer[128];
		FILE							*fp_play, *fp_record;
	#endif
#endif

	while(!pPtr->iExit) 
	{
		iDone=FALSE;
		mstime1=mstime2=0;
		waittimeout=0;
		memset(&timebuffer, 0, sizeof(struct _timeb));

		pPtr->LogPrint("READY FOR DTMF ON THREAD[%d]", DTMFData->ThreadNum);

		pPtr->Lock();
		DTMFData->ThreadWorking=0;
		pPtr->UnLock();
				
		hEvents[0] = DTMFData->StartEventHandle;

		dw = WaitForMultipleObjects(1, hEvents, 0, INFINITE);

		//hEvents[1] = DTMFData->ExitEventHandle;
		//dw = WaitForMultipleObjects(2, hEvents, 0, INFINITE);
		//if(dw == (WAIT_OBJECT_0 + 1)) continue;

		waittimeout=(DWORD) DTMFData->DTMFDetectTimeout;
			
		pPtr->LogPrint("STARTING DTMF ON[%d]", DTMFData->ThreadNum);

		/* Allocate OUTPUT Channel */
		INIT_ACU_SM_STRUCT(&sChannelParms_1);
		sChannelParms_1.type = kSMChannelTypeOutput;
		sChannelParms_1.module = pPtr->pModuleData[0].module_id;
		rc=sm_channel_alloc_placed(&sChannelParms_1);
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_alloc_placed", rc);

		/* Allocate INPUT Channel */
		INIT_ACU_SM_STRUCT(&sChannelParms_2);
		sChannelParms_2.type = kSMChannelTypeInput;
		sChannelParms_2.module = pPtr->pModuleData[0].module_id;
		rc=sm_channel_alloc_placed(&sChannelParms_2);
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_alloc_placed", rc);

		// Get Datafeed from Output Channel
		INIT_ACU_SM_STRUCT( &chan_datafeed );
		chan_datafeed.channel = sChannelParms_1.channel;
		rc = sm_channel_get_datafeed( &chan_datafeed );
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_get_datafeed", rc);

		// Connect Datafeed to Input Channel
		INIT_ACU_SM_STRUCT( &feed_connect );
		feed_connect.data_source	= chan_datafeed.datafeed;
		feed_connect.channel		= sChannelParms_2.channel;
		rc = sm_channel_datafeed_connect( &feed_connect );
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_datafeed_connect", rc);
			
		// Is this Detection or Generation
		if(DTMFData->DTMFDetect)
		{
			// Need a Play ( OUTPUT ) Channel event for sending Data to Detector
			INIT_ACU_SM_STRUCT(&sEventParmsPlay);
			sEventParmsPlay.channel=sChannelParms_1.channel;
			sEventParmsPlay.event_type=kSMEventTypeWriteData;
			sEventParmsPlay.issue_events=kSMChannelSpecificEvent;
			smd_ev_create(&sEventParmsPlay.event,sEventParmsPlay.channel,
						sEventParmsPlay.event_type,sEventParmsPlay.issue_events	);
			sm_channel_set_event(&sEventParmsPlay);

			// Need a DTMF ( INPUT ) Channel RECOG event for Detect
			INIT_ACU_SM_STRUCT(&sEventParmsDTMF);
			sEventParmsDTMF.channel=sChannelParms_2.channel;
			sEventParmsDTMF.event_type=kSMEventTypeRecog;
			sEventParmsDTMF.issue_events=kSMChannelSpecificEvent;
			smd_ev_create(&sEventParmsDTMF.event,sEventParmsDTMF.channel,
						sEventParmsDTMF.event_type,sEventParmsDTMF.issue_events	);
			sm_channel_set_event(&sEventParmsDTMF);

			// Listen for DTMF
			INIT_ACU_SM_STRUCT(&sListenParms);
			sListenParms.channel = sChannelParms_2.channel;
			sListenParms.tone_detection_mode = kSMToneDetectionMinDuration64; 
			sListenParms.active_tone_set_id = 0;
			sListenParms.map_tones_to_digits = kSMDTMFToneSetDigitMapping;		
			sListenParms.enable_cptone_recognition = 0;
			sListenParms.enable_grunt_detection = 0;
			rc = sm_listen_for(&sListenParms);
			if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_listen_for", rc);
		
#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
			/* Start Playing Prompt */
			INIT_ACU_SM_STRUCT(&sReplayParms);
			fp_play = fopen("dtmf.raw", "rb");
			if (fp_play == NULL)	printf("ERROR: Unable to open a.raw\n");
	#endif
#endif		

			INIT_ACU_SM_STRUCT(&sReplayParms);
			sReplayParms.channel = sChannelParms_1.channel;
			sReplayParms.type	= kSMDataFormat8KHzULawPCM;
			rc = sm_replay_start(&sReplayParms);
			if (rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_replay_start", rc);

			pPtr->LogPrint("PLAYING ON DTMF THREAD[%d]", DTMFData->ThreadNum);

		}
		else
		{
			// Need a Record ( INPUT ) Channel event for receiving Data from Generator
			INIT_ACU_SM_STRUCT(&sEventParmsRecord);
			sEventParmsRecord.channel=sChannelParms_2.channel;
			sEventParmsRecord.event_type=kSMEventTypeReadData;
			sEventParmsRecord.issue_events=kSMChannelSpecificEvent;
			smd_ev_create(&sEventParmsRecord.event,sEventParmsRecord.channel,
						sEventParmsRecord.event_type,sEventParmsRecord.issue_events	);
			sm_channel_set_event(&sEventParmsRecord);

			// Need a DTMF ( OUTPUT ) Channel WRITE event for Generation
			INIT_ACU_SM_STRUCT(&sEventParmsDTMF);
			sEventParmsDTMF.channel=sChannelParms_1.channel;
			sEventParmsDTMF.event_type=kSMEventTypeWriteData;
			sEventParmsDTMF.issue_events=kSMChannelSpecificEvent;
			smd_ev_create(&sEventParmsDTMF.event,sEventParmsDTMF.channel,
						sEventParmsDTMF.event_type,sEventParmsDTMF.issue_events	);
			sm_channel_set_event(&sEventParmsDTMF);

#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
			wsprintf(szBuffer, "c:/testdtmf%d.raw", DTMFData);
			fp_record = fopen(szBuffer, "wb");
			if(fp_record == NULL)	printf("ERROR: Unable to open test.raw\n");
	#endif
#endif		


			pPtr->LogPrint("RECORDING ON DTMF THREAD[%d]", DTMFData->ThreadNum);

			/* Start Recording Prompt */
			INIT_ACU_SM_STRUCT(&sRecordParms);
			sRecordParms.channel = sChannelParms_2.channel;
			sRecordParms.type = kSMDataFormat8KHzULawPCM;
			sRecordParms.max_octets = 0;
			sRecordParms.max_elapsed_time =	0;
			sRecordParms.max_silence = 0;

			rc=sm_record_start(&sRecordParms);
			if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_record_wav_start", rc);

		}

		// Is this Detection or Generation
		if(DTMFData->DTMFDetect)
		{
			x=0;

			/* Setup Waitable Objects */
			hEvents[0] = sEventParmsDTMF.event;
			hEvents[1] = sEventParmsPlay.event;
							
			while(!iDone)
			{
				_ftime( &timebuffer );
				mstime1=(timebuffer.time*1000)+timebuffer.millitm;

				//pPtr->LogPrint("MS1: %d, WTO: %d", mstime1, waittimeout);

				switch(WaitForMultipleObjects(2, hEvents, 0, waittimeout))
				{
					case WAIT_OBJECT_0:

						INIT_ACU_SM_STRUCT(&sRecogParms);
						sRecogParms.channel = sChannelParms_2.channel;
						rc = sm_get_recognised(&sRecogParms);

						if(sRecogParms.type == kSMRecognisedDigit)
						{
							DTMFData->DTMFDetectDigits[x++] = (char) sRecogParms.param0;
							pPtr->LogPrint("DTMF: %c", sRecogParms.param0);
						}

						waittimeout=DTMFData->DTMFDetectTimeout;

						//pPtr->LogPrint("DWTO: %d", waittimeout);
						
					break;


					/* Playback Data Required */
					case WAIT_OBJECT_0+1:

						_ftime( &timebuffer );
						mstime2=(timebuffer.time*1000)+timebuffer.millitm;
						waittimeout=(DWORD) ((DTMFData->DTMFDetectTimeout) - (mstime2-mstime1));

						//pPtr->LogPrint("MS2: %d, RWTO: %d", mstime2, waittimeout);
						
						INIT_ACU_SM_STRUCT(&sReplayStatusParms);
						sReplayStatusParms.channel = sChannelParms_1.channel;
						rc = sm_replay_status( &sReplayStatusParms );

						//pPtr->LogPrint("PLAY_ST:%d", sReplayStatusParms.status);

#if USE_SANGOMA
						if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
						{
							//receive data (on the way UP) FROM Sangoma interface
							rc = DTMFData->S->sangoma_rx(buffer, kSMMaxReplayDataBufferSize);
							sangoma_dbg_counter++;
							//printf("%d:sangoma returned %d bytes of data\n", sangoma_dbg_counter, rc);

							if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
							{
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms_1.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_last_replay_data( &sm_ts_data_parms );
								if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_put_last_replay", rc);
							} 
							else 
							{
								//  We got the expected amount of data from the file.
								//  Send it to Prosody
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms_1.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_replay_data( &sm_ts_data_parms );
							}
						}
						else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
						{
							//  Prosody has finished playing the file
							pPtr->LogPrint("FINISHED PLAYING ON DTMF THREAD [%d]", DTMFData->ThreadNum);
							iDone = TRUE;
						}

#else
	#if TEST_SANGOMA_STUBS

						if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
						{
							//receive data (on the way UP) FROM Sangoma interface
							rc = DTMFData->S->sangoma_rx(buffer, kSMMaxReplayDataBufferSize);

							if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
							{
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms_1.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_last_replay_data( &sm_ts_data_parms );
								if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_put_last_replay", rc);

							} 
							else 
							{
								//  We got the expected amount of data from the file.
								//  Send it to Prosody
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms_1.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_replay_data( &sm_ts_data_parms );
								if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_put_replay_data", rc);

							}
						}
						else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
						{
							//  Prosody has finished playing the file
							pPtr->LogPrint("FINISHED PLAYING ON DTMF THREAD [%d]", DTMFData->ThreadNum);
							iDone = TRUE;
						}
	#else

						if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
						{
							//  Channel has capacity for more data
							rc = fread( buffer, sizeof( char ), kSMMaxReplayDataBufferSize, fp_play );

							if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
							{
								//  Did not read as much data as we could
								if( ferror( fp_play ) )
								{
									fclose( fp_play );
									pPtr->ErrorPrint("DSPPrimitiveRecord", "ferror", rc);
								}
								else if( feof( fp_play ) )
								{
									//  End of file
									INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
									sm_ts_data_parms.channel = sChannelParms_1.channel;
									sm_ts_data_parms.data = buffer;
									sm_ts_data_parms.length = rc;
									rc = sm_put_last_replay_data( &sm_ts_data_parms );
									if(rc)
									{
										fclose( fp_play );
										pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_put_last_replay", rc);
									}
								}
							} 
							else 
							{
								//  We got the expected amount of data from the file.
								//  Send it to Prosody
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms_1.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_replay_data( &sm_ts_data_parms );
							}
						} 
						else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
						{
							//  Prosody has finished playing the file
							pPtr->LogPrint("FINISHED PLAYING ON DTMF THREAD [%d]", DTMFData->ThreadNum);
							iDone = TRUE;
						}
	#endif
#endif		
					break;

					case WAIT_TIMEOUT:
						DTMFData->DTMFDetectDigits[x]='\0';
						pPtr->LogPrint("TIMEOUT ON DTMF THREAD [%d]", DTMFData->ThreadNum);
						sPlayAbortParms.channel = sChannelParms_1.channel;
						rc = sm_replay_abort(&sPlayAbortParms);
					break;
										
					default:
					break;
				}
			}
		}
		else // DTMF GENERATION
		{
			/* Setup Waitable Objects */
			hEvents[0] = sEventParmsDTMF.event;
			hEvents[1] = sEventParmsRecord.event;

			/*** Generate DTMF ***/
			INIT_ACU_SM_STRUCT(&sPlayDTMF);
			sPlayDTMF.channel = sChannelParms_1.channel;
			sPlayDTMF.wait_for_completion = 0; // Dont' Block
			sPlayDTMF.digits.type = kSMDTMFDigits;
			strcpy(sPlayDTMF.digits.digit_string, DTMFData->DTMFGenerateDigits);
			sPlayDTMF.digits.inter_digit_delay = 100;
			sPlayDTMF.digits.digit_duration = 200;
			rc = sm_play_digits(&sPlayDTMF);
			if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_play_digits", rc);

			while(!iDone)
			{
				switch(WaitForMultipleObjects(2, hEvents, 0, 1000))
				{
					case WAIT_OBJECT_0:

						INIT_ACU_SM_STRUCT(&sStatusParms);
						sStatusParms.channel = sChannelParms_1.channel;
   						rc = sm_play_digits_status(&sStatusParms);
						if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_play_digits_status", rc);
						
						if(sStatusParms.status == kSMPlayDigitsStatusOngoing)
						{
						
						}
						// Write Event Fired ... Done Playing Digits
						else if(sStatusParms.status == kSMPlayDigitsStatusComplete)
						{
							pPtr->LogPrint("DONE GENERATING DIGITS ON DTMF THREAD [%d]", DTMFData->ThreadNum);
							sRecordAbortParms.channel = sChannelParms_2.channel;
							rc = sm_record_abort(&sRecordAbortParms);
						}
					break;

					/* Recording Data Required */
					case WAIT_OBJECT_0+1:

						INIT_ACU_SM_STRUCT(&sRecordStatus);
						sRecordStatus.channel = sChannelParms_2.channel;
						rc = sm_record_status(&sRecordStatus);
						if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_record_status", rc);

						// pPtr->LogPrint("REC_ST:%d", sRecordStatus.status);

						if(sRecordStatus.status == kSMRecordStatusComplete)
						{
							pPtr->LogPrint("FINISHED RECORDING ON DTMF THREAD[%d]", DTMFData->ThreadNum);
							iDone=TRUE;
						}
						else if(sRecordStatus.status == kSMRecordStatusData)
						{
							
							INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
							
							sm_ts_data_parms.channel	= sChannelParms_2.channel;
							sm_ts_data_parms.data		= buffer;
							
							rc = sm_get_recorded_data( &sm_ts_data_parms );
#if USE_SANGOMA
							if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_get_recorded_data", rc);
								
							//Transmit DOWN TO Sangoma interface. Make sure MTU/MRU of Sangoma interface
							//is configred to be 2048 bytes because Aculab exepcts it to be 2048 bytes.
							DTMFData->S->sangoma_tx(buffer, sm_ts_data_parms.length);
#else
	#if TEST_SANGOMA_STUBS
							if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_get_recorded_data", rc);
							DTMFData->S->sangoma_tx(buffer, sm_ts_data_parms.length);
	#else

							if(rc)
							{
								pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_get_recorded_data", rc);
								fclose( fp_record );
							}

							//  Write data to file
							fwrite( sm_ts_data_parms.data, sizeof( char ), sm_ts_data_parms.length, fp_record );
	#endif
#endif

						}
						
					break;

					default:
					break;
				}
			}
		}


		if(DTMFData->DTMFDetect)
		{

#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
			fclose( fp_play );
	#endif
#endif	
		
			smd_ev_free(sEventParmsPlay.event);
			smd_ev_free(sEventParmsDTMF.event);
		}
		else
		{
#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
			fclose( fp_record );
	#endif
#endif	

			smd_ev_free(sEventParmsDTMF.event);
			smd_ev_free(sEventParmsRecord.event);
		}
		
		/* Free up Prosody Resource */
		rc=sm_channel_release(sChannelParms_1.channel);
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_release", rc);

		/* Free up Prosody Resource */
		rc=sm_channel_release(sChannelParms_2.channel);
		if(rc) pPtr->ErrorPrint("ProsodyDTMFHandlerThread", "sm_channel_release", rc);
					
		pPtr->LogPrint("END OF SPEECH ON DTMF THREAD [%d]", DTMFData->ThreadNum);

		if(DTMFData->DTMFDetect) pPtr->DoDTMFDetectedCallback(DTMFData);
	}

	// SetEvent(ChanData->DoneEventHandle);	
	
	return 0;

}

DTMFClass::DTMFClass()
{
	int i;

	if(!init_once)
	{
		InitializeCriticalSection(&DTMF_CS);
		init_once=1;
	}
	
	/* Allocate DTMF Data Array */
	pDTMFData = (DTMF_DATA*) calloc(NUMCALLSPERPORT, sizeof(DTMF_DATA));
	memset(pDTMFData,0,unPorts*sizeof(DTMF_DATA));

	for(i=0; i < NUMCALLSPERPORT ; i++)
	{
		pDTMFData[i].StartEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		pDTMFData[i].ThreadNum = i;
		StartThread(i);
	}

	class_type = ACU_DTMF;
}

DTMFClass::~DTMFClass()
{
	if(init_once)
	{
		DeleteCriticalSection(&DTMF_CS);
		init_once=0;
	}
}

void DTMFClass::StartThread(int i)
{
	DWORD			dwThreadID;
	THREAD_DATA*	ThreadData;
	
	ThreadData = (THREAD_DATA*) calloc(1, sizeof(THREAD_DATA));
	memset(ThreadData,0,1*sizeof(THREAD_DATA));

	ThreadData->DCP = this;
	ThreadData->ts = i;

	HANDLE hProsodyThread = (HANDLE) CreateThread(NULL, 0, ProsodyDTMFHandlerThread, 
							 (void*) ThreadData, 0, &dwThreadID);
}

void DTMFClass::DoDTMFDetection(class sangoma_interface* so, long timeout)
{
	Lock();
	int			i;
	DTMF_DATA*	DTMFData;
	
	//Find a free DTMF Thread
	for(i=0; i < NUMCALLSPERPORT; i++)
	{
		DTMFData = NULL;
		DTMFData = &pDTMFData[i];

		if(!DTMFData->ThreadWorking)
		{
			memset(&DTMFData->DTMFDetectDigits, 0, sizeof(DTMFData->DTMFDetectDigits));
			DTMFData->ThreadWorking=1;
			DTMFData->S = so;
			DTMFData->DTMFDetect=1; // Do Detection
			DTMFData->DTMFDetectTimeout=timeout;
			SetEvent(DTMFData->StartEventHandle);
			break;
		}
	}
	UnLock();
}

void DTMFClass::DoDTMFGeneration(char* DTMFDigits, class sangoma_interface* so)
{
	Lock();
	int			i;
	DTMF_DATA*	DTMFData;
	
	//Find a free DTMF Thread
	for(i=0; i < NUMCALLSPERPORT; i++)
	{
		DTMFData = NULL;
		DTMFData = &pDTMFData[i];

		if(!DTMFData->ThreadWorking)
		{
			DTMFData->ThreadWorking=1;
			strcpy(DTMFData->DTMFGenerateDigits, DTMFDigits);
			DTMFData->S = so;
			DTMFData->DTMFDetect=0; // Do Generation
			SetEvent(DTMFData->StartEventHandle);
			break;
		}
	}
	UnLock();
}

void DTMFClass::SetDTMFDetectedCallback(void (*fp)(char*,class sangoma_interface*))
{
	fpDTMFDetectedCallback = fp;	
}

void DTMFClass::DoDTMFDetectedCallback(DTMF_DATA* DTMFData)
{
	(*fpDTMFDetectedCallback)(DTMFData->DTMFDetectDigits, DTMFData->S);
}
