//*******************************************************************
//
// Company:		Aculab, Inc.
//
// Author:		Jeff Feddon
//
// FileName:	IncomingClass.cpp 
//
// Date:		04/13/2007
//
// Description:	Wrapper class for Aculab Incoming Call API
//
//******************************************************************

#include "IncomingClass.h"
#include "AculabClass.h"



typedef struct _Thread_Data_
{
	IncomingClass* ICP;
	int			   ts;
} THREAD_DATA;

DWORD WINAPI IncomingSipPortEventManagerThread(LPVOID pParam)
{
	IncomingClass	*pPtr = (IncomingClass*) pParam;

	PORT_DATA* PortData = ( PORT_DATA* ) &pPtr->pPortData[0];
	
	int								numcalls = NUMCALLSPERPORT;
	int								ts,xx=0;
	long							mask = 1L;
	int								Done = 0;
	ACU_ERR							rc;

	ACU_ALLOC_EVENT_QUEUE_PARMS		rAllocEventQueue;
	ACU_QUEUE_PARMS					cSetPortQueue;
	ACU_QUEUE_WAIT_OBJECT_PARMS		rWaitObject;
	ACU_EVENT_QUEUE_PARMS			rEventFromQueue;

	STATE_XPARMS					cCallEvent;
	CALL_DATA*						CallData = NULL;
	
	CAUSE_XPARMS					cCallRelease;
	CAUSE_XPARMS					cCallDisconnect;
	DWORD							dwResult;
	HANDLE*							pHandles;

	ACU_FREE_EVENT_QUEUE_PARMS		rFreeEventQueue;

	/* Create Event Queue */
	INIT_ACU_STRUCT(&rAllocEventQueue);
	rc = acu_allocate_event_queue(&rAllocEventQueue);
	if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "acu_allocate_event_queue", rc);

	PortData->in_queue_id = rAllocEventQueue.queue_id;
	
	/* Attach Queue to Port */
	INIT_ACU_CL_STRUCT(&cSetPortQueue);
	cSetPortQueue.queue_id = PortData->in_queue_id;
	cSetPortQueue.resource_id = PortData->port_id;
	rc = call_set_port_default_handle_event_queue(&cSetPortQueue);
	if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "call_set_port_default_handle_event_queue", rc);

	/* Get a Queue Wait Object */
	INIT_ACU_STRUCT(&rWaitObject);
	rWaitObject.queue_id = PortData->in_queue_id;
	rc = acu_get_event_queue_wait_object(&rWaitObject);
	if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "acu_get_event_queue_wait_object", rc);

	while(!Done)
	{
		dwResult = WaitForSingleObject(rWaitObject.wait_object, 1000);

		if(dwResult == WAIT_OBJECT_0)
		{
			EnterCriticalSection(&PortData->IN_CS);
			/* Get Event From Queue */
			INIT_ACU_STRUCT(&rEventFromQueue);
			rEventFromQueue.queue_id = PortData->in_queue_id;
			rc = acu_get_event_from_queue(&rEventFromQueue);
			if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "acu_get_event_from_queue", rc);

			/* Dereference Context Token to Get the Struct Address */
			CallData = (CALL_DATA*) rEventFromQueue.context;

			if(CallData == NULL ) {/* fatal error */ exit(1);}
			
			/* Get the Event */
			INIT_ACU_CL_STRUCT(&cCallEvent);
			cCallEvent.handle = CallData->handle;
			rc = call_event(&cCallEvent);
			if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "call_event", rc);
			LeaveCriticalSection(&PortData->IN_CS);

			// pPtr->LogPrint("\nCALL STATE: %d", cCallEvent.state);

			switch(cCallEvent.state)
			{
				case EV_WAIT_FOR_INCOMING:
					pPtr->LogPrint("EV_WAIT_FOR_INCOMING [%d:%d]",CallData->port,CallData->ts);
					break;
				
				case EV_INCOMING_CALL_DET:
					pPtr->LogPrint("EV_INCOMING_CALL_DET [%d:%d]",CallData->port,CallData->ts);
					
					rc = pPtr->handle_ev_incoming_call_det(cCallEvent.handle, CallData->ts, &CallData->offer);
					if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "handle_ev_incoming_call_det", rc);

					// CallBack Function
					pPtr->DoIncomingDetectedCallback(CallData);
					
					break;

				case EV_WAIT_FOR_ACCEPT:
					// Ringing sent so...waiting for Rx event to be signalled 
				 	pPtr->LogPrint("\nEV_WAIT_FOR_ACCEPT found on port [%d], channel [%d]",CallData->port,CallData->ts);
					break;

				case EV_MEDIA:
					pPtr->LogPrint("EV_MEDIA [%d:%d]",CallData->port,CallData->ts);
					rc = pPtr->handle_ev_media(cCallEvent.handle, CallData->connected, &CallData->sip_details_parms);
					if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "handle_ev_media", rc);

					// Signal Speech Thread to set TX and RX codecs etc...
					SetEvent(CallData->ConfigureRTPEventHandle);
					
					break;
				
				case EV_CALL_CONNECTED:
					pPtr->LogPrint("EV_CALL_CONNECTED [%d:%d]",CallData->port,CallData->ts);
					rc = pPtr->handle_ev_call_connected(cCallEvent.handle);
					if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "handle_ev_call_connected", rc);

					SetEvent(CallData->ConnectedEventHandle);

					// CallBack Function
					pPtr->DoIncomingConnectedCallback(CallData);
										
					break;

				case EV_REMOTE_DISCONNECT:
					pPtr->LogPrint("EV_REMOTE_DISCONNECT [%d:%d]",CallData->port,CallData->ts);
					INIT_ACU_CL_STRUCT(&cCallDisconnect);
					cCallDisconnect.handle = cCallEvent.handle;
					rc = call_disconnect(&cCallDisconnect);
					if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "call_disconnect", rc);
					break;
				
				case EV_IDLE:
					pPtr->LogPrint("EV_IDLE [%d:%d]",CallData->port,CallData->ts);

					INIT_ACU_CL_STRUCT(&cCallRelease);
					cCallRelease.handle = cCallEvent.handle;
					rc = call_release(&cCallRelease);
					if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "call_release", rc);

					CallData->connected = 0;
					CallData->opened = 0;	

					PortData->calls_in_progress--;

					if(!pPtr->iExit)
					{
						SetEvent(CallData->IdleEventHandle);
						pPtr->DoIncomingIdleCallback(CallData);
					}
					else if(!PortData->calls_in_progress) Done++;

					
										
					break;
					
			}	

		}

		/* APP EXIT SIGNALLED...THEN DISCONNECT ALL CALLS */
		if(!Done)
		{
			if(pPtr->iExit)
			{
				EnterCriticalSection(&PortData->IN_CS);
				for(ts=0;ts<numcalls;ts++)
				{
					CallData = &PortData->pBChanDataIN[ts];
					if((CallData->connected || CallData->opened)&&(!CallData->disconnected))
					{
						INIT_ACU_CL_STRUCT(&cCallDisconnect);
						cCallDisconnect.handle = CallData->handle;
						rc = call_disconnect(&cCallDisconnect);
						if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "call_disconnect", rc);
						pPtr->LogPrint("Call Disconnected [%d:%d]",CallData->port, CallData->ts);
						CallData->disconnected=1;

					}
				}
				LeaveCriticalSection(&PortData->IN_CS);
			}
		}

	}

	/* Free Event Queue */
	INIT_ACU_STRUCT(&rFreeEventQueue);
	rFreeEventQueue.queue_id = PortData->in_queue_id;
	rc = acu_free_event_queue(&rFreeEventQueue);
	if(rc) pPtr->ErrorPrint("SipPortEventManagerThread", "acu_free_event_queue", PortData->port);

	/* Allocate Handle Array */
	pHandles = (HANDLE*) calloc(numcalls, sizeof(HANDLE));
	memset(pHandles, 0,numcalls*sizeof(HANDLE));

	for(ts=0;ts<numcalls;ts++)
	{
		CallData = &PortData->pBChanDataIN[ts];
		
		// In case Thread is Waiting...Tell Thread to Exit
		SetEvent(CallData->ExitEventHandle);
		
		// Get an Event for Notification Thread is Done
		pHandles[ts]=CallData->DoneEventHandle;

	}

	/* Wait for Speech Threads to Terminate */
	dwResult = WaitForMultipleObjects(numcalls, pHandles, 1, INFINITE);

	free( pHandles );

	/* Send signal that Port Thread is terminating */
	SetEvent(PortData->done_event);

	pPtr->LogPrint("END OF CALL EVENTS ON PORT %d", PortData->port);

	return 1;

}

IncomingClass::IncomingClass()
{
	//LogPrint("IncomingConstructor");
	DWORD dwThreadID;
	HANDLE hSipThread = (HANDLE) CreateThread(NULL, 0, IncomingSipPortEventManagerThread, 
								 (void*) this, 0, &dwThreadID);
	
	// Give Thread Time to start up before servicing call states
	Sleep(1000);

	class_type = ACU_INCOMING;
}

IncomingClass::~IncomingClass()
{

}

DWORD WINAPI IncomingProsodyChannelHandlerThread(LPVOID pParam)
{
	THREAD_DATA *TD = (THREAD_DATA*) pParam;
	IncomingClass	*pPtr = (IncomingClass*) TD->ICP;
	int i = TD->ts;
	free(TD);

	PORT_DATA* PortData = ( PORT_DATA* ) &pPtr->pPortData[0];
	CALL_DATA* ChanData = ( CALL_DATA* ) &PortData->pBChanDataIN[i];

	int									rc;
	int									iDone, iGotPort, iDonePlaying, iDoneRecording;
	int									Count = 0;
	DWORD								dw;
	char								buffer[kSMMaxReplayDataBufferSize];		

	SM_CHANNEL_ALLOC_PLACED_PARMS		sChannelParms;

	SM_RECORD_PARMS						sRecordParms;
	SM_RECORD_ABORT_PARMS				sRecordAbortParms;
	SM_RECORD_STATUS_PARMS				sRecordStatus;

	SM_REPLAY_STATUS_PARMS				sReplayStatusParms;
	SM_REPLAY_PARMS						sReplayParms;
	SM_REPLAY_ABORT_PARMS				sPlayAbortParms;
	SM_TS_DATA_PARMS					sm_ts_data_parms;

	SM_CHANNEL_SET_EVENT_PARMS			sEventParmsRecord;
	SM_CHANNEL_SET_EVENT_PARMS			sEventParmsPlay;
	SM_VMPRX_PORT_PARMS					rx_port_info;
	HANDLE								hEvents[3];
	HANDLE								hDummy = CreateEvent(NULL,FALSE,FALSE,NULL);

#if USE_SANGOMA
	static int							sangoma_dbg_counter = 0;
#else
	#if TEST_SANGOMA_STUBS

	#else
		char							szBuffer[128];
		FILE							*fp_play, *fp_record;
	#endif
#endif

	while(!pPtr->iExit)
	{
		iDone=iGotPort=iDoneRecording=iDonePlaying=FALSE;

		pPtr->LogPrint("READY FOR SPEECH ON[%d:%d]", ChanData->port, ChanData->ts);

		if(ChanData->IsSIPCall)
		{

			// Wait to Reserve RTP until Incoming Call Detected
			hEvents[0] = ChanData->ReserveRTPEventHandle;
			hEvents[1] = ChanData->ExitEventHandle;
			
			dw = WaitForMultipleObjects(2, hEvents, 0, INFINITE);

			if(dw == (WAIT_OBJECT_0 + 1)) continue;
			
			pPtr->Reserve_RTP(ChanData);

			// Wait until Got RTP Port
			hEvents[0] = ChanData->vmptx_event;
			hEvents[1] = ChanData->vmprx_event;
					
			while(!iGotPort) 
			{
				dw = WaitForMultipleObjects(2, hEvents, 0, 100);

				if(dw == WAIT_OBJECT_0)
				{ 
					rc = pPtr->Get_RTP_TX_Status(ChanData);
				}
				else if( dw == (WAIT_OBJECT_0 + 1))
				{
					rc = pPtr->Get_RTP_RX_Status(ChanData);
					if(rc == kSMVMPrxStatusGotPorts)
					{

						INIT_ACU_SM_STRUCT( &rx_port_info );
						rx_port_info.vmprx	= ChanData->vmp_rx.vmprx;
						rx_port_info.nowait = 1;

						rc = sm_vmprx_get_ports( &rx_port_info );
						if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_vmp_rx_get_ports", rc);	
						
						ChanData->rtp_port = rx_port_info.RTP_port;				

						// Send 200 OK
						rc = pPtr->handle_sip_accept(ChanData);
						if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sip_accept_call", rc);	
						
						iGotPort=1;
					}
				}
			}

			hEvents[0] = ChanData->ConfigureRTPEventHandle;
			hEvents[1] = ChanData->ExitEventHandle;
			
			// Wait for EV_MEDIA
			dw = WaitForMultipleObjects(2, hEvents, 0, INFINITE);

			if(dw == (WAIT_OBJECT_0 + 1)) 
			{	
				pPtr->Stop_RTP(ChanData); 
				iDone=TRUE; 
				continue;
			}
			else 
			{
				pPtr->Configure_RTP(ChanData, &ChanData->sip_details_parms);							
				rc = sip_free_details(&ChanData->sip_details_parms);
				if(rc) pPtr->ErrorPrint("SIPProsodyChannelHandlerThread", "sip_free_details", rc);
			}
			
		}

		hEvents[0] = ChanData->ConnectedEventHandle;
		hEvents[1] = ChanData->ExitEventHandle;
				
		// Wait for Call Connected
		dw = WaitForMultipleObjects(2, hEvents, 0, INFINITE);

		if(dw == (WAIT_OBJECT_0 + 1)) 
		{
			if(ChanData->IsSIPCall) pPtr->Stop_RTP(ChanData); 
			iDone=TRUE; 
			continue;
		}
	
		pPtr->LogPrint("STARTING SPEECH ON[%d:%d]", ChanData->port, ChanData->ts);

		/* Allocate Channel */
		INIT_ACU_SM_STRUCT(&sChannelParms);
		sChannelParms.type = kSMChannelTypeFullDuplex;
		sChannelParms.module = pPtr->pModuleData[0].module_id;
		rc=sm_channel_alloc_placed(&sChannelParms);
		if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_channel_alloc_placed", rc);
			
		ChanData->TiNG_ChannelId = sChannelParms.channel;

		/* Register Channel for Record */
		INIT_ACU_SM_STRUCT(&sEventParmsRecord);
		sEventParmsRecord.channel=sChannelParms.channel;
		sEventParmsRecord.event_type=kSMEventTypeReadData;
		sEventParmsRecord.issue_events=kSMChannelSpecificEvent;
		smd_ev_create(&sEventParmsRecord.event,sEventParmsRecord.channel,
					sEventParmsRecord.event_type,sEventParmsRecord.issue_events	);
		sm_channel_set_event(&sEventParmsRecord);

		/* Register Channel for Play */
		INIT_ACU_SM_STRUCT(&sEventParmsPlay);
		sEventParmsPlay.channel=sChannelParms.channel;
		sEventParmsPlay.event_type=kSMEventTypeWriteData;
		sEventParmsPlay.issue_events=kSMChannelSpecificEvent;
		smd_ev_create(&sEventParmsPlay.event,sEventParmsPlay.channel,
					sEventParmsPlay.event_type,sEventParmsPlay.issue_events	);
		sm_channel_set_event(&sEventParmsPlay);

		// Switch RTP to TiNG Resource ( TX and RX )
		pPtr->Connect_RTP_to_TiNG(ChanData);
			
		pPtr->LogPrint("PLAYING ON[%d:%d]", ChanData->port, ChanData->ts);

#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
		/* Start Playing Prompt */
		INIT_ACU_SM_STRUCT(&sReplayParms);
		fp_play = fopen("in.raw", "rb");
		if (fp_play == NULL)	printf("ERROR: Unable to open a.raw\n");
	#endif
#endif		
		INIT_ACU_SM_STRUCT(&sReplayParms);
		sReplayParms.channel = sChannelParms.channel;
		sReplayParms.type	= kSMDataFormat8KHzULawPCM;
		rc = sm_replay_start(&sReplayParms);
		if (rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_replay_start", rc);

		pPtr->LogPrint("RECORDING ON[%d:%d]", ChanData->port, ChanData->ts);

#if USE_SANGOMA

#else
	#if TEST_SANGOMA_STUBS

	#else
		wsprintf(szBuffer, "c:/testin%d.raw", ChanData);
		fp_record = fopen(szBuffer, "wb");
		if(fp_record == NULL)	printf("ERROR: Unable to open test.raw\n");
	#endif
#endif
		
		/* Start Recording Prompt */
		INIT_ACU_SM_STRUCT(&sRecordParms);
		sRecordParms.channel = sChannelParms.channel;
		sRecordParms.type = kSMDataFormat8KHzULawPCM;
		sRecordParms.max_octets = 0;
		sRecordParms.max_elapsed_time =	0;
		sRecordParms.max_silence = 0;

		rc=sm_record_start(&sRecordParms);
		if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_record_wav_start", rc);

		/* Setup Waitable Objects */
		hEvents[0] = sEventParmsPlay.event;
		hEvents[1] = sEventParmsRecord.event;
		hEvents[2] = ChanData->IdleEventHandle;
				
		while(!iDone)
		{
			switch(WaitForMultipleObjects(3, hEvents, 0, 100))
			{
				/* Playback Data Required */
				case WAIT_OBJECT_0:
				
				INIT_ACU_SM_STRUCT(&sReplayStatusParms);
				sReplayStatusParms.channel = sChannelParms.channel;
				rc = sm_replay_status( &sReplayStatusParms );

				//pPtr->LogPrint("PLAY_ST:%d", sReplayStatusParms.status);

#if USE_SANGOMA
				if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
				{
					//Receive data (on the way UP) FROM Sangoma Analog interface
					rc = ChanData->S->sangoma_rx(buffer, kSMMaxReplayDataBufferSize);
					sangoma_dbg_counter++;
					//DBG_IFACE("%d:sangoma returned %d bytes of data\n", sangoma_dbg_counter, rc);

					if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
					{
						INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
						sm_ts_data_parms.channel = sChannelParms.channel;
						sm_ts_data_parms.data = buffer;
						sm_ts_data_parms.length = rc;
						rc = sm_put_last_replay_data( &sm_ts_data_parms );
						if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_put_last_replay", rc);
					} 
					else 
					{
						//  We got the expected amount of data from the file.
						//  Send it to Prosody
						INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
						sm_ts_data_parms.channel = sChannelParms.channel;
						sm_ts_data_parms.data = buffer;
						sm_ts_data_parms.length = rc;
						rc = sm_put_replay_data( &sm_ts_data_parms );
					}
				}
				else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
				{
					//  Prosody has finished playing the file
					pPtr->LogPrint("(1) FINISHED PLAYING ON[%d:%d]", ChanData->port, ChanData->ts);
					if(iDonePlaying) iDone = TRUE;
					else iDoneRecording++;
				}

#else
	#if TEST_SANGOMA_STUBS

					if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
					{
						//receive data (on the way UP) FROM Sangoma interface
						rc = ChanData->S->sangoma_rx(buffer, kSMMaxReplayDataBufferSize);

						if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
						{
							INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
							sm_ts_data_parms.channel = sChannelParms.channel;
							sm_ts_data_parms.data = buffer;
							sm_ts_data_parms.length = rc;
							rc = sm_put_last_replay_data( &sm_ts_data_parms );
							if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_put_last_replay", rc);

						} 
						else 
						{
							//  We got the expected amount of data from the file.
							//  Send it to Prosody
							INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
							sm_ts_data_parms.channel = sChannelParms.channel;
							sm_ts_data_parms.data = buffer;
							sm_ts_data_parms.length = rc;
							rc = sm_put_replay_data( &sm_ts_data_parms );
							if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_put_replay_data", rc);

						}
					}
					else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
					{
						//  Prosody has finished playing the file
						pPtr->LogPrint("(2) FINISHED PLAYING ON[%d:%d]", ChanData->port, ChanData->ts);
						if(iDonePlaying) iDone = TRUE;
						else iDoneRecording++;
					}
	#else

					if( sReplayStatusParms.status == kSMReplayStatusHasCapacity )
					{
						//  Channel has capacity for more data
						rc = fread( buffer, sizeof( char ), kSMMaxReplayDataBufferSize, fp_play );

						if( rc != sizeof( char ) * kSMMaxReplayDataBufferSize )
						{
							//  Did not read as much data as we could
							if( ferror( fp_play ) )
							{
								fclose( fp_play );
								pPtr->ErrorPrint("DSPPrimitiveRecord", "ferror", rc);
							}
							else if( feof( fp_play ) )
							{
								//  End of file
								INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
								sm_ts_data_parms.channel = sChannelParms.channel;
								sm_ts_data_parms.data = buffer;
								sm_ts_data_parms.length = rc;
								rc = sm_put_last_replay_data( &sm_ts_data_parms );
								if(rc)
								{
									fclose( fp_play );
									pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_put_last_replay", rc);
								}
							}
						} 
						else 
						{
							//  We got the expected amount of data from the file.
							//  Send it to Prosody
							INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
							sm_ts_data_parms.channel = sChannelParms.channel;
							sm_ts_data_parms.data = buffer;
							sm_ts_data_parms.length = rc;
							rc = sm_put_replay_data( &sm_ts_data_parms );
						}
					} 
					else if( sReplayStatusParms.status == kSMReplayStatusComplete ) 
					{
						//  Prosody has finished playing the file
						pPtr->LogPrint("(3) FINISHED PLAYING ON[%d:%d]", ChanData->port, ChanData->ts);

						if(iDonePlaying) iDone = TRUE;
						else iDoneRecording++;
					}
	#endif
#endif		
				break;

				/* Recording Data Required */
				case WAIT_OBJECT_0+1:

					INIT_ACU_SM_STRUCT(&sRecordStatus);
					sRecordStatus.channel = sChannelParms.channel;
					rc = sm_record_status(&sRecordStatus);
					if(rc) pPtr->ErrorPrint("RecordReplayThread", "sm_record_status", rc);

					//pPtr->LogPrint("REC_ST:%d", sRecordStatus.status);

					if(sRecordStatus.status == kSMRecordStatusComplete)
					{
						pPtr->LogPrint("FINISHED RECORDING ON[%d:%d]", ChanData->port, ChanData->ts);
						if(iDoneRecording) iDone=TRUE;
						else iDonePlaying++;
					}
					else if(sRecordStatus.status == kSMRecordStatusData)
					{
						INIT_ACU_SM_STRUCT(&sm_ts_data_parms);
						
						sm_ts_data_parms.channel	= sChannelParms.channel;
						sm_ts_data_parms.data		= buffer;
						
						rc = sm_get_recorded_data( &sm_ts_data_parms );

#if USE_SANGOMA
						if(rc) pPtr->ErrorPrint("RecordReplayThread", "sm_get_recorded_data", rc);
							
						//Transmit DOWN TO Sangoma interface. Make sure MTU/MRU of Sangoma interface
						//is configred to be 2048 bytes because Aculab exepcts it to be 2048 bytes.
						ChanData->S->sangoma_tx(buffer, sm_ts_data_parms.length);
#else
	#if TEST_SANGOMA_STUBS
						if(rc) pPtr->ErrorPrint("IncomingRecordReplayThread", "sm_get_recorded_data", rc);
						ChanData->S->sangoma_tx(buffer, sm_ts_data_parms.length);
	#else

						if(rc)
						{
							pPtr->ErrorPrint("RecordReplayThread", "sm_get_recorded_data", rc);
							fclose( fp_record );
						}

						//  Write data to file
						fwrite( sm_ts_data_parms.data, sizeof( char ), sm_ts_data_parms.length, fp_record );
	#endif
#endif

					}else{
						INFO_IFACE("line: %d\n", __LINE__);
						//pPtr->ErrorPrint("rc: %d\n", "sm_get_recorded_data", rc);
					}
					
				break;

				/* Call Disconnected */
				case WAIT_OBJECT_0+2:
					sPlayAbortParms.channel = sChannelParms.channel;
					rc = sm_replay_abort(&sPlayAbortParms);
					sRecordAbortParms.channel = sChannelParms.channel;
					rc = sm_record_abort(&sRecordAbortParms);
				break;

				default:
				break;
			}
		}

		// Break RTP <--> TiNG Connection
		pPtr->Disconnect_RTP_from_TiNG(ChanData);

		// Stop RTP
		pPtr->Stop_RTP(ChanData);
		
		smd_ev_free(sEventParmsRecord.event);
		smd_ev_free(sEventParmsPlay.event);
		
		/* Free up Prosody Resource */
		rc=sm_channel_release(sChannelParms.channel);
		if(rc) pPtr->ErrorPrint("ProsodyChannelHandlerThread", "sm_channel_release", rc);
			
		pPtr->LogPrint("END OF SPEECH ON[%d:%d]", ChanData->port, ChanData->ts);
	}

	SetEvent(ChanData->DoneEventHandle);	
	
	return 0;

}

int  IncomingClass::Get_RTP_TX_Status(CALL_DATA* CallData)
{
	int							rc;
	SM_VMPTX_STATUS_PARMS		statusp_tx;

	INIT_ACU_SM_STRUCT( &statusp_tx );
	statusp_tx.vmptx = CallData->vmp_tx.vmptx;
	rc = sm_vmptx_status(&statusp_tx);

	if(rc) 
		ErrorPrint("Get_RTP_TX_Status", "sm_vmptx_status", rc);	
	else if (statusp_tx.status == kSMVMPtxStatusStopped) 
	{
		//	printf("\nVMP_TX: STOPPED\n" );
		rc = sm_vmptx_destroy(CallData->vmp_tx.vmptx);
		if(rc) ErrorPrint("Get_RTP_TX_Status", "sm_vmptx_destroy", rc);	
	}	
	else if( statusp_tx.status == kSMVMPtxStatusRunning )
	{
	//	printf("\nVMP_TX: STARTED ");
	}	

	return statusp_tx.status;
}

int  IncomingClass::Get_RTP_RX_Status(CALL_DATA* CallData)
{
	int							rc;
	SM_VMPRX_STATUS_PARMS		statusp_rx;

	INIT_ACU_SM_STRUCT( &statusp_rx );
	statusp_rx.vmprx = CallData->vmp_rx.vmprx;
	rc = sm_vmprx_status(&statusp_rx);

	if(rc) 
		ErrorPrint("Get_RTP_RX_Status", "sm_vmprx_status", rc);	
	else if (statusp_rx.status == kSMVMPrxStatusStopped) 
	{
		//printf("\nVMP_RX: STOPPED\n" );
		rc = sm_vmprx_destroy(CallData->vmp_rx.vmprx);
	}
	else if ( statusp_rx.status == kSMVMPrxStatusGotPorts )
	{
		//printf("\nGot Port");
	}
	else if( statusp_rx.status == kSMVMPrxStatusDetectTone )
	{
		// printf( "\nVMP_RX: TONE DETECT" );		
		LogPrint("\n[%d][%d] -- RFC 2833 = %d",CallData->port,CallData->ts, statusp_rx.u.tone.id);
	}
	else if( statusp_rx.status == kSMVMPrxStatusRunning )
	{
		//printf( "\nVMP_RX: STARTED" );
	}

	return statusp_rx.status;
}

int IncomingClass::Reserve_RTP(CALL_DATA* ChanData)
{
	int				rc;		

	SM_VMPTX_EVENT_PARMS	evp_tx;
	SM_VMPRX_EVENT_PARMS	evp_rx;

	INIT_ACU_SM_STRUCT( &ChanData->vmp_rx );
	ChanData->vmp_rx.module = pModuleData[ChanData->module].module_id;
	rc = sm_vmprx_create( &ChanData->vmp_rx );
	if(rc) ErrorPrint("Reserve_VMP_Context", "sm_vmprx_create", rc);

	INIT_ACU_SM_STRUCT( &evp_rx );
	evp_rx.vmprx = ChanData->vmp_rx.vmprx;
	rc = sm_vmprx_get_event(&evp_rx);
	if(rc) ErrorPrint("Reserve_VMP_Context", "sm_vmprx_get_event", rc);

	ChanData->vmprx_event = evp_rx.event;
		
	INIT_ACU_SM_STRUCT( &ChanData->vmp_tx );
	ChanData->vmp_tx.module = pModuleData[ChanData->module].module_id;
	rc = sm_vmptx_create( &ChanData->vmp_tx );
	if(rc) ErrorPrint("Reserve_VMP_Context", "sm_vmptx_create", rc);
	INIT_ACU_SM_STRUCT( &evp_tx );
	evp_tx.vmptx = ChanData->vmp_tx.vmptx;
	rc = sm_vmptx_get_event(&evp_tx);
	if(rc) ErrorPrint("Reserve_VMP_Context", "sm_vmptx_get_event", rc);

	ChanData->vmptx_event = evp_tx.event;

	// printf("\nVMPRX EVENT: %x....VMPTX EVENT: %x", evp_rx.event, evp_tx.event);

	ChanData->rtp_port = 0;
	return 0;
}

void IncomingClass::Stop_RTP(CALL_DATA* ChanData)
{
	int									rc, iDone = 0, Count = 0;
	DWORD								dw;

	HANDLE								hEvents[2];
	HANDLE								hDummy = CreateEvent(NULL,FALSE,FALSE,NULL);
	
	SM_VMPTX_STOP_PARMS					vmptx_stop_parms;
	SM_VMPRX_STOP_PARMS					vmprx_stop_parms;

	INIT_ACU_SM_STRUCT( &vmptx_stop_parms );
	vmptx_stop_parms.vmptx = ChanData->vmp_tx.vmptx;
	rc = sm_vmptx_stop( &vmptx_stop_parms );
	if(rc) ErrorPrint("Stop_RTP", "sm_vmptx_stop", rc);

	INIT_ACU_SM_STRUCT( &vmprx_stop_parms );
	vmprx_stop_parms.vmprx = ChanData->vmp_rx.vmprx;
	rc = sm_vmprx_stop( &vmprx_stop_parms );
	if(rc) ErrorPrint("Stop_RTP", "sm_vmprx_stop", rc);

	hEvents[0] = ChanData->vmptx_event;
	hEvents[1] = ChanData->vmprx_event;

	while(!iDone) 
	{
		dw = WaitForMultipleObjects(2, hEvents, 0, 100);

		if(dw == WAIT_OBJECT_0)
		{ 
			rc = Get_RTP_TX_Status(ChanData); 
			
			if(rc == kSMVMPrxStatusStopped)
			{
				if(Count) iDone++; else Count++;
				hEvents[0] = hDummy;
			}
		}
		else if( dw == (WAIT_OBJECT_0 + 1))
		{
			rc = Get_RTP_RX_Status(ChanData); 
			
			if(rc == kSMVMPrxStatusStopped) 
			{
				if(Count) iDone++; else Count++;
				hEvents[1] = hDummy;
			}
		}
	}
}

void IncomingClass::Configure_RTP(CALL_DATA* ChanData, SIP_DETAIL_PARMS* sip_details_parms)
{
	SM_VMPTX_CONFIG_PARMS	config_vmptx;
	SM_VMPTX_CODEC_PARMS	tx_codec;
	SM_VMPRX_CODEC_PARMS	rx_codec;
	SM_VMPRX_TONE_PARMS		config_rx_tones;
	int						rc;

	INIT_ACU_SM_STRUCT( &config_vmptx );
	config_vmptx.vmptx								= ChanData->vmp_tx.vmptx;
	config_vmptx.destination_rtp.sin_addr.s_addr	= inet_addr( sip_details_parms->media_session->received_media.connection_address.address );
	config_vmptx.destination_rtp.sin_port			= htons( sip_details_parms->media_session->received_media.media_descriptions->port );
	config_vmptx.destination_rtcp.sin_addr.s_addr	= inet_addr( sip_details_parms->media_session->received_media.connection_address.address );
	config_vmptx.destination_rtcp.sin_port			= htons(sip_details_parms->media_session->received_media.media_descriptions->port+1);
	config_vmptx.source_rtp.sin_port				= htons(sip_details_parms->media_session->sent_media.media_descriptions->port);
	config_vmptx.source_rtp.sin_addr.s_addr			= 0;
	
	rc = sm_vmptx_config( &config_vmptx );
	if(rc) ErrorPrint("Configure_vmptx_and_vmprx", "sm_vmptx_config", rc);

	INIT_ACU_SM_STRUCT( &tx_codec );

	tx_codec.vmptx			= ChanData->vmp_tx.vmptx;
	tx_codec.codec			= convert_payload_number_to_ting( sip_details_parms->media_session->sent_media.media_descriptions->payloads->payload.audio_video.rtp_payload_number );
	tx_codec.payload_type	= sip_details_parms->media_session->sent_media.media_descriptions->payloads->payload.audio_video.rtp_payload_number;

	//need to check if this is set in the incoming SIP call
	if( sip_details_parms->media_session->sent_media.media_descriptions->packet_length != 0 )
	{
		tx_codec.ptime = sip_details_parms->media_session->sent_media.media_descriptions->packet_length ;
	}
	else
	{
		//a default value
		tx_codec.ptime = 20;
	}

	rc = sm_vmptx_config_codec( &tx_codec );
	if(rc) ErrorPrint("Configure_vmptx_and_vmprx", "sm_vmptx_config_codec", rc);

	INIT_ACU_SM_STRUCT( &rx_codec );

	//need to convert from the payload into the correct TiNG enum for the codec.....
	rx_codec.vmprx			= ChanData->vmp_rx.vmprx;
	rx_codec.codec			= convert_payload_number_to_ting( sip_details_parms->media_session->sent_media.media_descriptions->payloads->payload.audio_video.rtp_payload_number );
	rx_codec.payload_type	= sip_details_parms->media_session->sent_media.media_descriptions->payloads->payload.audio_video.rtp_payload_number;

	rc = sm_vmprx_config_codec( &rx_codec );
	if(rc) ErrorPrint("Configure_vmptx_and_vmprx", "sm_vmprx_config_codec", rc);

	if( sip_details_parms->media_session->sent_media.media_descriptions->payloads->next )
	{
		if( 0 == strcmp( sip_details_parms->media_session->sent_media.media_descriptions->payloads->next->payload.audio_video.rtp_payload_name, ACU_TELEPHONE_EVENT ) )
		{
			rx_codec.codec			= kSMCodecTypeRFC2833;
			rx_codec.payload_type	= sip_details_parms->media_session->sent_media.media_descriptions->payloads->next->payload.audio_video.rtp_payload_number;
			rc = sm_vmprx_config_codec( &rx_codec );	
			if(rc) ErrorPrint("Configure_vmptx_and_vmprx", "sm_vmprx_config_codec", rc);
		}
	}

	INIT_ACU_SM_STRUCT( &config_rx_tones );

	config_rx_tones.vmprx			= ChanData->vmp_rx.vmprx;
	config_rx_tones.detect_tones	= 1;
	config_rx_tones.regen_tones		= 1;

	rc = sm_vmprx_config_tones( &config_rx_tones );
	if(rc) ErrorPrint("Configure_vmptx_and_vmprx", "sm_vmprx_config_tones", rc);

}

void IncomingClass::Connect_RTP_to_TiNG(CALL_DATA* ChanData)
{
	int									rc;
	SM_CHANNEL_DATAFEED_PARMS			chan_datafeed;
	SM_VMPTX_DATAFEED_CONNECT_PARMS		datafeed_connect;
	SM_VMPRX_DATAFEED_PARMS				chan_feed;
	SM_CHANNEL_DATAFEED_CONNECT_PARMS	feed_connect;

	INIT_ACU_SM_STRUCT( &chan_datafeed );
	chan_datafeed.channel = ChanData->TiNG_ChannelId;
	rc = sm_channel_get_datafeed( &chan_datafeed );
	if(rc) ErrorPrint("Connect_RTP_to_TiNG", "sm_channel_get_datafeed", rc);

	INIT_ACU_SM_STRUCT( &datafeed_connect );
	datafeed_connect.vmptx			= ChanData->vmp_tx.vmptx;
	datafeed_connect.data_source	= chan_datafeed.datafeed;
	rc = sm_vmptx_datafeed_connect( &datafeed_connect );
	if(rc) ErrorPrint("Connect_RTP_to_TiNG", "sm_vmptx_datafeed_connect", rc);

	INIT_ACU_SM_STRUCT( &chan_feed );
	chan_feed.vmprx = ChanData->vmp_rx.vmprx;
	rc = sm_vmprx_get_datafeed( &chan_feed );
	if(rc) ErrorPrint("Connect_RTP_to_TiNG", "sm_vmprx_get_datafeed", rc);

	//printf("\nIN_DATAFEED_CONNECT");
	//printf("...DF: %x...TC: %x...");
	INIT_ACU_SM_STRUCT( &feed_connect );
	feed_connect.data_source	= chan_feed.datafeed;
	feed_connect.channel		= ChanData->TiNG_ChannelId;
	rc = sm_channel_datafeed_connect( &feed_connect );
	if(rc) ErrorPrint("Connect_RTP_to_TiNG", "sm_channel_datafeed_connect", rc);
	//printf("\nOUT_DATAFEED_CONNECT");
}

void IncomingClass::Disconnect_RTP_from_TiNG(CALL_DATA* ChanData)
{
	int									rc;
	SM_VMPTX_DATAFEED_CONNECT_PARMS		con_datafeed;
	SM_CHANNEL_DATAFEED_CONNECT_PARMS	feed_connect;
	
	INIT_ACU_SM_STRUCT( &con_datafeed );
	INIT_ACU_SM_STRUCT( &feed_connect );
	
	con_datafeed.vmptx			= ChanData->vmp_tx.vmptx;
	con_datafeed.data_source	= kSMNullDatafeedId ;
	rc = sm_vmptx_datafeed_connect( &con_datafeed );
	if(rc) ErrorPrint("Disconnect_RTP_to_TiNG", "sm_vmptx_datafeed_connect", rc);
	
	feed_connect.channel		= ChanData->TiNG_ChannelId;
	feed_connect.data_source	= kSMNullDatafeedId;
	rc = sm_channel_datafeed_connect( &feed_connect );
	if(rc) ErrorPrint("Disconnect_RTP_to_TiNG", "sm_channel_datafeed_connect", rc);
}

enum kSMCodecType IncomingClass::convert_payload_number_to_ting( ACU_INT payload_number )
{
	switch( payload_number )
	{
	case ACU_PCMU_PAYLOAD_NUMBER: return kSMCodecTypeMulaw;
	case ACU_PCMA_PAYLOAD_NUMBER: return kSMCodecTypeAlaw;
	case ACU_G729_PAYLOAD_NUMBER: return kSMCodecTypeG729AB;
	default: return ( enum kSMCodecType ) -1;
	}

}

ACU_CALL_HANDLE IncomingClass::OpenIncoming(int ts)
{
	PORT_DATA* PortData = ( PORT_DATA* ) &pPortData[0];

	int								rc;
	CALL_DATA*						CallDataIn;
	SIP_IN_PARMS					cOpenIn;
	THREAD_DATA*					ThreadData;

	CallDataIn = NULL;
	CallDataIn = &PortData->pBChanDataIN[ts];

	EnterCriticalSection(&PortData->IN_CS);
	/* Open Call for Incoming */
	INIT_ACU_CL_STRUCT(&cOpenIn);
	cOpenIn.net = PortData->port_id;
	cOpenIn.queue_id = PortData->in_queue_id;
	cOpenIn.app_context_token = (ACU_ACT) CallDataIn;
	rc = sip_openin(&cOpenIn);
	if(rc) ErrorPrint("SipPortEventManagerThread", "sip_openin", rc);

	if(!CallDataIn->init_once)
	{
		CallDataIn->ts = ts;
		CallDataIn->port = PortData->port;
		CallDataIn->IsSIPCall = 1;
		CallDataIn->IdleEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		CallDataIn->ConnectedEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		CallDataIn->ReserveRTPEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		CallDataIn->ConfigureRTPEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);	
		CallDataIn->DoneEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		CallDataIn->ExitEventHandle = CreateEvent(NULL,FALSE,FALSE,NULL);
		CallDataIn->module = 0;

		DWORD dwThreadID;
		
		ThreadData = (THREAD_DATA*) calloc(1, sizeof(THREAD_DATA));
		memset(ThreadData,0,1*sizeof(THREAD_DATA));

		ThreadData->ICP = this;
		ThreadData->ts = ts;
		
		HANDLE hProsodyThread = (HANDLE) CreateThread(NULL, 0, IncomingProsodyChannelHandlerThread, 
								 (void*) ThreadData, 0, &dwThreadID);

		CallDataIn->init_once=1;
	}

	CallDataIn->handle = cOpenIn.handle;
	CallDataIn->opened = 1;
	PortData->calls_in_progress++;

	LeaveCriticalSection(&PortData->IN_CS);

	return (ACU_CALL_HANDLE) cOpenIn.handle;

}

ACU_ERR IncomingClass::handle_ev_incoming_call_det(const ACU_CALL_HANDLE handle, int ts, ACU_MEDIA_OFFER_ANSWER** offer)
{
   ACU_ERR rc = 0;
   SIP_DETAIL_PARMS details;
   // SIP_INCOMING_RINGING_PARMS sip_incoming_ringing_parms;

   // Retrieve the media offer in the SIP_DETAIL_PARMS structure.
   INIT_ACU_CL_STRUCT(&details);
   details.handle = handle;
   rc = sip_details(&details);
   if(0 != rc)
   {
      printf("Error in sip_details(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   // Get Destination Number ... SIP:DESTNUM@IPADDRESS

   char					newstring[20];
   char					string[30];
   char					*ps = string;
   char					*pns = newstring;

   strcpy(string, details.destination_addr);  
   while((*ps++ != ':') && (*ps)) { }
   while((*ps != '@') && (*ps)) 
      *pns++ = *ps++; 
   *pns = '\0';

   	CALL_DATA*	CallData;

	CallData = NULL;
	CallData = &pPortData[0].pBChanDataIN[ts];

	strcpy(CallData->dest, newstring);

   // print_tabbed_raw_sdp(details.media_offer_answer->raw_sdp);
   // Copy the media offer.
   *offer = clone_media_offer(details.media_offer_answer);

   // Free the SIP_DETAIL_PARMS structure.
   rc = sip_free_details(&details);
   if(0 != rc)
   {
      printf("Error in sip_free_details(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   // INIT_ACU_CL_STRUCT(&sip_incoming_ringing_parms);
   // sip_incoming_ringing_parms.handle = handle;
   // printf("\tsip_incoming_ringing()...\n\n");
   // Alert the call - sends a 180 Ringing response to the far end with no SDP body.
   // rc = sip_incoming_ringing(&sip_incoming_ringing_parms);
   if(0 != rc)
   {
      printf("Error in sip_incoming_ringing(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   return 0;
}

ACU_ERR IncomingClass::handle_sip_accept(CALL_DATA* ChanData)
{
   ACU_ERR rc = 0;
   SIP_ACCEPT_PARMS accept;
   ACU_MEDIA_OFFER_ANSWER *tempAnswer;
  
   // Create a media answer using the previously stored offer.
   tempAnswer = populate_media_answer(ChanData->offer);
   if(NULL == tempAnswer)
   {
      // Reject call here as no supported codec has been found.
   }

   // Copy media answer to SIP_ACCEPT_PARMS structure
   // Accept the call - sends a 200 OK response to the far end with SDP body.
   // printf("\nsip_accept()...MEDIA ANSWER:\n");
   INIT_ACU_CL_STRUCT(&accept);
   accept.handle = ChanData->handle;
   accept.media_offer_answer = *tempAnswer;
   accept.media_offer_answer.media_descriptions->port = ChanData->rtp_port;
   //strcpy(accept.media_offer_answer.media_descriptions->connection_address.address, card_ip_address);
   rc = sip_accept(&accept);
   if(0 != rc)
   {
      printf("Error in sip_accept(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   // print_media_offer(tempAnswer);

   // Make sure that all dynamically allocated memory is released.
   free_media_offer(&tempAnswer);
   free_media_offer(&ChanData->offer);
   return 0;
}

ACU_ERR IncomingClass::handle_ev_media(const ACU_CALL_HANDLE handle, const ACU_INT is_reinvite, SIP_DETAIL_PARMS* sip_details_parms)
{
   ACU_ERR rc = 0;

   CAUSE_XPARMS cause;
   
   // Retrieve the media details in the SIP_DETAIL_PARMS structure.
   INIT_ACU_CL_STRUCT(sip_details_parms);
   sip_details_parms->handle = handle;
   rc = sip_details(sip_details_parms);
   if(0 != rc)
   {
      printf("Error in sip_details(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   if(is_reinvite)
   {
      // printf("EV_MEDIA\n\tAt this point the application should re-configure\n\tthe RTP stream with the new settings.");
      if(0 == call_handle_2_io(handle))
      {
         printf("\n\tPress return to disconnect the call.\n");
         getchar();

		 INIT_ACU_CL_STRUCT(&cause);
		 cause.handle = handle;
		 rc = call_disconnect(&cause);
		 if(0 != rc)
		 {
			printf("Error in call_disconnect(): %d, %s\n", rc, error_2_string(rc));
			return rc;
		 }

      }
   }
   else
   {
      // printf("EV_MEDIA\n\tAt this point the application should configure and\n\tstart the RTP stream.\n");
   }

   return 0;
}

ACU_ERR IncomingClass::handle_ev_call_connected(const ACU_CALL_HANDLE handle) 
{
   ACU_ERR rc = 0;
   SIP_DETAIL_PARMS sip_details_parms;

   INIT_ACU_CL_STRUCT(&sip_details_parms);
   sip_details_parms.handle = handle;
   rc = sip_details(&sip_details_parms);
   if(0 != rc)
   {
      printf("Error in sip_details(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   rc = sip_free_details(&sip_details_parms);
   if(0 != rc)
   {
      printf("Error in sip_free_details(): %d, %s\n", rc, error_2_string(rc));
      return rc;
   }

   return 0;
}

ACU_ERR IncomingClass::handle_ev_idle(const ACU_CALL_HANDLE handle)
{
    ACU_ERR rc = 0;
    CAUSE_XPARMS release;

    printf("\nEV_IDLE\n\tHere the application should stop the RTP stream and\n\trelease the call handle.\n\n");
    INIT_ACU_CL_STRUCT(&release);
    release.handle = handle;
    rc = call_release(&release);
    if(0 != rc)
    {
        printf("Error in call_release(): %d, %s\n", rc, error_2_string(rc));
        return rc;
    }

    return 0;
}

void IncomingClass::SetIncomingDetectedCallback(void (*fp)(ACU_CALL_HANDLE,int,char*))
{
	fpIncomingDetectedCallback = fp;	
}

void IncomingClass::SetIncomingConnectedCallback(void (*fp)(ACU_CALL_HANDLE,int))
{
	fpIncomingConnectedCallback = fp;	
}

void IncomingClass::SetIncomingIdleCallback(void (*fp)(ACU_CALL_HANDLE,int))
{
	fpIncomingIdleCallback = fp;	
}

void IncomingClass::Send200OK(int ts)
{
	CALL_DATA*	CallData;

	CallData = NULL;
	CallData = &pPortData[0].pBChanDataIN[ts];

	// Signal Speech Thread to Allocate RTP and Send 200OK
	SetEvent(CallData->ReserveRTPEventHandle);
}

void IncomingClass::DoIncomingDetectedCallback(CALL_DATA* CallData)
{
	(*fpIncomingDetectedCallback)(CallData->handle, CallData->ts, CallData->dest);
}

void IncomingClass::DoIncomingConnectedCallback(CALL_DATA* CallData)
{
	(*fpIncomingConnectedCallback)(CallData->handle, CallData->ts);
}

void IncomingClass::DoIncomingIdleCallback(CALL_DATA* CallData)
{
	(*fpIncomingIdleCallback)(CallData->handle, CallData->ts);
}

void IncomingClass::DisconnectCall(int ts)
{
	int				rc;
	CALL_DATA*		CallData;
	CAUSE_XPARMS	cCallDisconnect;

	DBG_IFACE("IncomingClass::DisconnectCall()\n");

	CallData = NULL;
	CallData = &pPortData[0].pBChanDataIN[ts];

	INIT_ACU_CL_STRUCT(&cCallDisconnect);
	cCallDisconnect.handle = CallData->handle;
	rc = call_disconnect(&cCallDisconnect);
	if(rc) ErrorPrint("DisconnnectCall", "call_disconnect", rc);
}

void IncomingClass::SetSangomaObject(class sangoma_interface* so, int ts)
{
	CALL_DATA*	CallData;
	CallData = NULL;
	CallData = &pPortData[0].pBChanDataIN[ts];
	CallData->S = so;
}

void *IncomingClass::GetSangomaObject(int ts)
{
	CALL_DATA*	CallData;
	CallData = NULL;
	CallData = &pPortData[0].pBChanDataIN[ts];
	return CallData->S;
}
