// data_queue.h: interface for the data_queue class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DATA_QUEUE_H__D9043556_AC28_403B_8C82_690BE8668B6A__INCLUDED_)
#define AFX_DATA_QUEUE_H__D9043556_AC28_403B_8C82_690BE8668B6A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <stdlib.h>

///////////////////////////////////////////////////////////////////////////
//
//structures and definitions used for queueing data
//
typedef struct
{
	void*				previous;
	TX_RX_DATA_STRUCT	tx_rx_data;
}api_queue_element_t;

#define API_Q_MAX_SIZE		10	//optimal length. for short data may need longer queue

enum API_Q_STATUS{
	API_Q_SUCCESS=0,
	API_Q_MEM_ALLOC_FAILURE,
	API_Q_FULL,
	API_Q_EMPTY
};

class data_queue  
{
	//number of nodes in the list
	USHORT size;
	//insert at tail
	api_queue_element_t * tail;
	//remove from head
	api_queue_element_t * head;

	//mutex for synchronizing access to the queue
	CRITICAL_SECTION	queue_cs;

public:
	data_queue()
	{
		size = 0;
		tail = head = NULL;
		InitializeCriticalSection(&queue_cs);
	}

	~data_queue()
	{
		DeleteCriticalSection(&queue_cs);
	}

	int api_enqueue(unsigned char * buffer,	unsigned int length)
	{
		api_queue_element_t *element;

		EnterCriticalSection(&queue_cs); 

		if(size == API_Q_MAX_SIZE){
			printf("queue is full!\n");
			LeaveCriticalSection(&queue_cs);
			return API_Q_FULL;
		}

		element = (api_queue_element_t *)malloc(sizeof(api_queue_element_t));
		if(element == NULL){
			printf("failed allocate memory for new queue element!\n");
			LeaveCriticalSection(&queue_cs);
			return API_Q_MEM_ALLOC_FAILURE;
		}

		//now copy everything in to the element
		memcpy(element->tx_rx_data.data, buffer, length);
		
		element->tx_rx_data.api_header.data_length = length;
		element->tx_rx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;

		//insert element at the tail of the queue
		element->previous = NULL;

		if(size == 0){
			//special case of a previously empty queue
			head = element;
			tail = element;
		}else{
			tail->previous = element;
			tail = element;
		}
		size++;
		LeaveCriticalSection(&queue_cs);
		return API_Q_SUCCESS;
	}

	int api_dequeue(TX_RX_DATA_STRUCT* destination)
	{
		api_queue_element_t *element;

		EnterCriticalSection(&queue_cs); 

		if(size == 0){
			//tx queue is empty
			LeaveCriticalSection(&queue_cs);
			return API_Q_EMPTY;
		}

		//remove from the head of the queue
		element = head;
		head = (api_queue_element_t *)element->previous;

		//now copy everything in to the user buffer
		memcpy(destination, &element->tx_rx_data, sizeof(TX_DATA_STRUCT));

		free(element);
		size--;
		if(size == 0){
			head = NULL;
			tail = NULL;
		}
		LeaveCriticalSection(&queue_cs);
		return API_Q_SUCCESS;
	}
/*
	//remove all elements from the queue
	void empty_api_queue()
	{
		TX_DATA_STRUCT tx_rx_data;

		printf("empty_api_queue()\n");
		while(api_dequeue(&tx_rx_data) == 0){
			;
		}
	}
*/
};

#endif // !defined(AFX_DATA_QUEUE_H__D9043556_AC28_403B_8C82_690BE8668B6A__INCLUDED_)
