//*******************************************************************
//
// Company:		Sangoma Technologies, Inc.
//
// Author:		David Rokhvarg <davidr@sangoma.com>
//
// FileName:	sangoma_interface.h
//
// Description:	encapsulates capabilities of a Sangoma API interface
//
// Version:		1.0		05/7/2007	initial version
//
//******************************************************************

#ifndef SANGOMA_INTERFACE_H
#define SANGOMA_INTERFACE_H

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()


#define __WINDOWS__
#include <wanpipe_defines.h>	//for 'wan_udp_hdr_t'
#include <sang_status_defines.h>//return codes
#include <sang_api.h>			//for IOCTL codes
#include <sdla_remora.h>		//Analog card
#include <wanpipe_cfg.h>		//for RBS commands, Media Types (WAN_MEDIA_NONE...)

#include <wanpipe_tdm_api.h>	//for TDMV API
#include <sdla_aft_te1.h>		//for Wanpipe API

#include "data_queue.h"

//#include <stelephony_exports.h>

#define ERR_IFACE	if(1)printf
#define WRN_IFACE	if(1)printf
#define DBG_IFACE	if(1)printf
#define INFO_IFACE	if(1)printf

#define WAIT Sleep

#define RING_DELAY()		Sleep(1000)
#define RING_STOP_DELAY()	Sleep(2000)

typedef struct{
	//this function will be called if Analog side accepted incoming SIP call.
	int		(*sangoma_accepted_incoming_sip_call)(void *aculab_object, int timeslot);

	//this function will be called if Analog side wants to make outgoing SIP call.
	int		(*sangoma_outgoing_sip_call)(void *sangoma_object);

	//this function will be called if an Analog Phone, which is connected to Sangoma FXO went On Hook.
	int		(*sangoma_on_hook)(void *sangoma_object);

}sangoma_callback_functions_t;

/**
  *@author David Rokhvarg
  */

struct func_protocol {
	int		protocol_id;
	unsigned char   prot_name[10];
	/*
	config_t*	config;
	usage_t*	usage;
	main_t*		main;
	dis_trace_t*	disable_trace;
	main_menu_t*	get_main_menu;
	cmd_menu_t*	get_cmd_menu;
	ft1_cfg_t*	read_FT1_status;
	ft1_cfg_t*	set_FT1_mode;
	*/
	unsigned char	mbox_offset;
};


#define DECODE_PROT(prot)((prot==WANCONFIG_FR)?"Frame Relay":	\
			  (prot==WANCONFIG_MFR)?"Frame Relay":				\
	          (prot==WANCONFIG_PPP)?"PPP Point-to-Point":		\
	          (prot==WANCONFIG_CHDLC)?"Cisco HDLC":				\
	          (prot==WANCONFIG_X25)?"X25":						\
	          (prot==WANCONFIG_ADSL)?"ADSL: Eth,IP,PPP/ATM":	\
			  (prot==WANCONFIG_ATM)?"ATM":						\
			  (prot==WANCONFIG_AFT)?"AFT T1/E1":				\
			  (prot==WANCONFIG_AFT_TE1)?"AFT T1/E1":			\
			  (prot==WANCONFIG_AFT_ANALOG)?"AFT Analog":		\
							"Unknown")


#define DEV_NAME_LEN			100
//Aculab expects audio stream in 2048 long bytes buffers.
#define AUDIO_DATA_BUFFER_SIZE	2048

#define REAL_BUFFER_SIZE	1024

typedef enum _SANG_IF_STATE{
	IF_FREE=1,	//interface is not doing anything
	IF_BUSY,	//call is active
	IF_WAITING_FOR_ANALOG_TO_PICKUP,
	IF_WAITING_FOR_SIP_TO_PICKUP,
	IF_NO_FREE_SIP_TIMESLOTS_FOR_CALLING_OUT

}SANG_IF_STATE;

class sangoma_interface {

	

	//////////////////////////////////////////////////////////////////
	HANDLE		hGeneralCommands;
	HANDLE		hRx;
	HANDLE		hTx;
	HANDLE		hEvents;

	//////////////////////////////////////////////////////////////////
	//receive stuff
	ULONG		rx_frames_count;
	ULONG		rx_bytes_count;
	//for counting frames with CRC/Abort errors
	ULONG		bad_rx_frames_count;
	DWORD		dwRxThreadId; 

	//////////////////////////////////////////////////////////////////
	//transmit stuff
	ULONG		tx_bytes_count;
	ULONG		tx_frames_count;
	UCHAR		tx_test_byte;

	//////////////////////////////////////////////////////////////////
	//IOCTL management structures and variables
	wan_udp_hdr_t	wan_udp;
	int				protocol_cb_size;
	int 			wan_protocol;
	unsigned char	adapter_type;

	wanpipe_tdm_api_cmd_t tdm_api_cmd;

	void PrintRxData(RX_DATA_STRUCT *pRx);
	void PrintTdmApiEvent(RX_DATA_STRUCT *pRx);
	void cleanup();

	wan_cmd_api_t			wanpipe_api_cmd;
	api_tx_hdr_t			*tx_hdr;

	int get_interface_configuration(if_cfg_t *wanif_conf_ptr);
	int GetWANConfig( void );
	unsigned char get_wan_udphdr_data_byte(unsigned char off);
	unsigned char *get_wan_udphdr_data_ptr(unsigned char off);
	unsigned char set_wan_udphdr_data_byte(unsigned char off, unsigned char data);
	int get_fe_type(unsigned char* adapter_type);

	int wanpipe_api_ioctl(wan_cmd_api_t *api_cmd);

	USHORT	DoReadCommand(HANDLE drv, RX_DATA_STRUCT * pRx);
	UCHAR	DoWriteCommand(HANDLE drv, TX_DATA_STRUCT * pTx);
	UCHAR	DoSetIdleTxBufferCommand(HANDLE drv, TX_DATA_STRUCT *pTx);
	USHORT	DoReadEventCommand(HANDLE drv, RX_DATA_STRUCT * pRx);

	unsigned char wp_brt[256]; 

	void generate_bit_rev_table(void)
	{
		unsigned char util_char;
		unsigned char misc_status_byte;
		int i;

		/* generate the bit-reversing table for all unsigned characters */
		for(util_char = 0;; util_char ++) {
			misc_status_byte = 0;			/* zero the character to be 'built' */
			/* process all 8 bits of the source byte and generate the */
			for(i = 0; i <= 7; i ++) {
		      		/* corresponding 'bit-flipped' character */
				if(util_char & (1 << i)) {
					misc_status_byte |= (1 << (7 - i));
				}
			}
			/* insert the 'bit-flipped' character into the table 
			 * at the appropriate location */
			wp_brt[util_char] = misc_status_byte; 

			/* exit when all unsigned characters have been processed */
			if(util_char == 0xFF) {
				break;
			}
		}
	}

	SANG_IF_STATE	if_state;

	void	*aculab_obj;
	int		 aculab_timeslot;


	data_queue  rx_q;
	data_queue  tx_q;

	int stop();

	int create_ring_thread();

#define IP_ADDR_LENGTH	16
	char outgoing_ip_address[IP_ADDR_LENGTH*2];

#define MAX_CID_LEN		100
	//buffer for Caller ID incoming from SIP.
	char in_cid[MAX_CID_LEN];

	//buffer for Caller ID incoming from Analog.
	char out_cid[MAX_CID_LEN];

public:

	int ring_thread_running;

	int read_event();
	int rx_thread_running;

	void set_aculab_obj(void *a_obj)
	{
		DBG_IFACE("%s:set_aculab_obj()\n", device_name);
		aculab_obj = a_obj;
	}

	void * get_aculab_obj()
	{
		DBG_IFACE("%s:get_aculab_obj()\n", device_name);
		return aculab_obj;
	}

	void set_aculab_timeslot(int ts)
	{
		DBG_IFACE("%s:set_aculab_timeslot()\n", device_name);
		aculab_timeslot = ts;
	}

	int get_aculab_timeslot()
	{
		DBG_IFACE("%s:get_aculab_timeslot()\n", device_name);
		return aculab_timeslot;
	}

	CRITICAL_SECTION	SangIfRxCriticalSection;
	CRITICAL_SECTION	SangIfTxCriticalSection;
	CRITICAL_SECTION	cs_tdmv_api_ioctl;
	CRITICAL_SECTION	cs_wanpipe_api_ioctl;

	//////////////////////////////////////////////////////////////////
	//data
	int						terminate_rx_thread;
	RX_DATA_STRUCT			rx_data;
	sangoma_callback_functions_t	callback_functions;
	unsigned char			last_hook_state_event;

	char					device_name[DEV_NAME_LEN];	
	if_cfg_t				wanif_conf_struct;

	int print_counter;

	//////////////////////////////////////////////////////////////////
	//methods
	
	sangoma_interface(){ printf("invalid constructor\n");}//FIXME: remove in finanl build

	sangoma_interface(int wanpipe_number, int interface_number);
	~sangoma_interface();

	int DoManagementCommand(HANDLE drv, wan_udp_hdr_t* packet);
#define DO_COMMAND(wan_udp)	DoManagementCommand(hGeneralCommands, &wan_udp);

	int init(sangoma_callback_functions_t *callback_functions_ptr);
	int run();
	int enable_events();
	int disable_events();

	void read_te1_56k_stat(void);
	void set_lb_modes(unsigned char type, unsigned char mode);
	
	void operational_stats (void);
	void flush_operational_stats (void);

	char get_rbs(rbs_management_t *rbs_management_ptr);
	char set_rbs(rbs_management_t *rbs_management_ptr);

	int get_open_handles_counter();

	HANDLE open_api_device(char *api_device_name);


	void decode_if_state(SANG_IF_STATE	st)
	{
		switch(st)
		{
		case IF_FREE:
			DBG_IFACE("IF_FREE\n");
			break;
		case IF_BUSY:
			DBG_IFACE("IF_BUSY\n");
			break;
		case IF_WAITING_FOR_ANALOG_TO_PICKUP:
			DBG_IFACE("IF_WAITING_FOR_ANALOG_TO_PICKUP\n");
			break;
		case IF_WAITING_FOR_SIP_TO_PICKUP:
			DBG_IFACE("IF_WAITING_FOR_SIP_TO_PICKUP\n");
			break;
		default:
			DBG_IFACE("Unknown IF state! (%d)\n", st);
			break;
		}
	}

	SANG_IF_STATE get_if_state(char *caller_name)
	{
#if 0
		DBG_IFACE("%s: get_if_state(): current state is: ", device_name);
		DBG_IFACE("caller_name: %s\n", caller_name);

		decode_if_state(if_state);
#endif
		return if_state;
	}

	unsigned int set_if_state(SANG_IF_STATE new_if_state)
	{
		DBG_IFACE("%s: set_if_state(): new state is: ", device_name);
		decode_if_state(new_if_state);

		if_state = new_if_state;

		return 0;
	}

	UCHAR stransmit(TX_DATA_STRUCT *pTx)
	{
		int rc;
		EnterCriticalSection(&SangIfTxCriticalSection); 
		rc = DoWriteCommand(hTx, pTx);
		LeaveCriticalSection(&SangIfTxCriticalSection);
		return rc;
	}

	//make sure only one thread at a time reads from the same device.
	USHORT sreceive(RX_DATA_STRUCT *pRx)
	{
		int rc=0;
		EnterCriticalSection(&SangIfRxCriticalSection); 
		rc = DoReadCommand(hRx, pRx);
		LeaveCriticalSection(&SangIfRxCriticalSection);
		return rc;
	}

	USHORT receive_event(RX_DATA_STRUCT *pRx)
	{
		int rc=0;
		rc = DoReadEventCommand(hEvents, pRx);
		return rc;
	}


	int sangoma_tx(void *source_buffer, int data_length)
	{
		TX_DATA_STRUCT local_tx_data;
#if defined(VOICE_APP)
		bit_swap_a_buffer(source_buffer, data_length);
#endif
		local_tx_data.api_header.data_length = data_length;
		local_tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;
		//set the actual data for transmission
		memcpy(local_tx_data.data, source_buffer, data_length);

		if(	stransmit(&local_tx_data)){
			//error
			ERR_IFACE("%s: sangoma_tx() failed!! Check messages log.\n", device_name);
			return 1;
		}
				
		//check that frame was transmitted
		switch(local_tx_data.api_header.operation_status)
		{
		case SANG_STATUS_SUCCESS:
			tx_bytes_count += local_tx_data.api_header.data_length;
			tx_frames_count++;
			//printf("Frame queued for transmission.\n");
			//signal to anyone who may wait for transmission completion
			break;
							
		case SANG_STATUS_TX_TIMEOUT:
			//error
			//It means that tx buffers are constanly full,
			//usually because the line is disconnected
			ERR_IFACE("****** SANG_STATUS_TX_TIMEOUT ******\n");
			//Check messages log or look at statistics.
			break;
							
		case SANG_STATUS_TX_DATA_TOO_LONG:
			//Attempt to transmit data longer than the pre-configured maximum.
			//Maximum length is set in 'Interface Properties',
			//in the 'Device Manager'.
			ERR_IFACE("****** SANG_STATUS_TX_DATA_TOO_LONG ******\n");
			break;
							
		case SANG_STATUS_TX_DATA_TOO_SHORT:
			//Minimum is 1 byte  for Primary   port,
			//			 2 bytes for Secondary port
			ERR_IFACE("****** SANG_STATUS_TX_DATA_TOO_SHORT ******\n");
			break;

		case SANG_STATUS_LINE_DISCONNECTED:
			//Front end monitoring is enabled and Line is
			//in disconnected state.
			//Check the T1/E1 line is in "Connected" state,
			//alse check the Alarms and the message log.
			printf("****** SANG_STATUS_LINE_DISCONNECTED ******\n");
			break;

		default:
			ERR_IFACE("Unknown return code (0x%X) on transmission!\n",
				local_tx_data.api_header.operation_status);
			break;

		}//switch()

		return 0; 
	}

	//this is used only for Audio Rx
	USHORT sangoma_rx(void *destination_buffer, int destination_buffer_length)
	{
		api_header_t		*pri;
		USHORT				rc;
		
		//DBG_IFACE("sangoma_rx(): %s\n", device_name);

		if(sreceive(&rx_data)){
			//error
			ERR_IFACE( "receive() failed!! Check messages log.\n");
			return 1;
		}

		//use our special buffer at rxdata to hold received data
		pri = &rx_data.api_header;

		rc = pri->operation_status;

		switch(pri->operation_status)
		{
		case SANG_STATUS_RX_DATA_TIMEOUT:
			//No data in READ_CMD_TIMEOUT, try again. (NOT and error)
			DBG_IFACE("Timeout on read.\n");
			break;

		case SANG_STATUS_BUFFER_TOO_SMALL:
			//Recieved data longer than the pre-configured maximum.
			//Maximum length is set in 'Interface Properties',
			//in the 'Device Manager'.
			ERR_IFACE( "Received data longer than buffer passed to API.\n");
			break;

		case SANG_STATUS_LINE_CONNECTED:
			INFO_IFACE( "%s: SANG_STATUS_LINE_CONNECTED\n", device_name);
			//Sleep(1000);//wait - to avoid busy loop
			break;

		case SANG_STATUS_LINE_DISCONNECTED:
			INFO_IFACE( "%s: SANG_STATUS_LINE_DISCONNECTED\n", device_name);
			//Sleep(1000);//wait - to avoid busy loop
			break;

		case SANG_STATUS_RX_DATA_AVAILABLE:

			if(AUDIO_DATA_BUFFER_SIZE != pri->data_length){
				ERR_IFACE( "%s: Rx: Invalid data length: %d!! (expected %d)\n", 
					device_name, pri->data_length, AUDIO_DATA_BUFFER_SIZE);
				rc = 0;
				break;
			}

#define RX_Q_TRESHOLD 2
			//If there is more than RX_Q_TRESHOLD buffers of Audio in Rx queue, the delay may get
			//noticeable. This can happen at the begining of the call and never after that.
			//Remove excessive data.
			if(rx_data.api_header.rx_h.current_number_of_frames_in_rx_queue > RX_Q_TRESHOLD){
				WRN_IFACE("%s: Warning: discarding %d buffers of Audio.\n", device_name,
					rx_data.api_header.rx_h.current_number_of_frames_in_rx_queue - RX_Q_TRESHOLD);

				while(rx_data.api_header.rx_h.current_number_of_frames_in_rx_queue > RX_Q_TRESHOLD){
					sreceive(&rx_data);
				}
			}

			//copy to Aculab buffer
			memcpy(destination_buffer, rx_data.data, destination_buffer_length);
			rc = AUDIO_DATA_BUFFER_SIZE;
			break;

		default:
			ERR_IFACE( "%s: Rx: Unknown Operation Status: %d\n", 
				device_name, pri->operation_status);
			break;
		}//switch()

		//If returning "rc != AUDIO_DATA_BUFFER_SIZE", Aculab stops the receiver.
		return AUDIO_DATA_BUFFER_SIZE;
	}


	UCHAR DoSetSharedEventCommand(PREGISTER_EVENT event)
	{
		DWORD ln;
	
		if(DeviceIoControl(
			hRx,
			IoctlSetSharedEvent,
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
			//check messages log
			return 1;
		}else{
			return 0;
		}
	}

	void bit_swap_a_buffer(unsigned char *data, int len)
	{
		int i;
		for (i=0; i < len; i++){
			data[i]=wp_brt[data[i]];
		}
	}

#define DEFAULT_OUTGOING_CALL_IP_ADDRESS	"192.168.1.125"

	int set_outgoing_ip_address()
	{
		int result = 0;
		int retry_counter = 0;

		INFO_IFACE("\n%s: Type IP address for Outgoing SIP calls and press <Enter>.\n For example: %s. (or 'd' and <Enter> for default)\n",
			device_name, DEFAULT_OUTGOING_CALL_IP_ADDRESS);

		while(scanf("%s", &outgoing_ip_address) == 0){
			fflush( stdin );
			INFO_IFACE("\nInvalid input! Must be an IP address. For example: %s\n",
				DEFAULT_OUTGOING_CALL_IP_ADDRESS);
			if(retry_counter++ > 5){
				INFO_IFACE("Giving up and using default IP: %s\n", DEFAULT_OUTGOING_CALL_IP_ADDRESS);
				snprintf((char*)outgoing_ip_address, IP_ADDR_LENGTH*2, DEFAULT_OUTGOING_CALL_IP_ADDRESS);
				result = 1;
				break;
			}
		}//while()

		if(strcmp(outgoing_ip_address, "d") == 0 || strcmp(outgoing_ip_address, "D") == 0){
			INFO_IFACE("Using default IP: '%s'\n", DEFAULT_OUTGOING_CALL_IP_ADDRESS);
			snprintf((char*)outgoing_ip_address, IP_ADDR_LENGTH*2, DEFAULT_OUTGOING_CALL_IP_ADDRESS);
		}

		printf("User input: '%s'\n", outgoing_ip_address);
		return result;
	}

	///////////////////////////////////////////////////////////////////////////
	char *get_outgoing_ip_address()
	{
		if(strlen((const char*)outgoing_ip_address) == 0){
			//outgoing IP is not initialized yet for this interface
			return NULL;
		}else{
			return (char*)outgoing_ip_address;
		}
	}
	
	///////////////////////////////////////////////////////////////////////////
	int set_incoming_caller_id(char *caller_id)
	{
		snprintf(in_cid, MAX_CID_LEN, caller_id);
		return 0;
	}

	char *get_incoming_caller_id()
	{
		return (char*)in_cid;
	}

	///////////////////////////////////////////////////////////////////////////
	int set_outgoing_caller_id(char *caller_id)
	{
		snprintf(out_cid, MAX_CID_LEN, caller_id);
		return 0;
	}

	char *get_outgoing_caller_id()
	{
		return (char*)out_cid;
	}

	///////////////////////////////////////////////////////////////////////////
	int tdmv_api_ioctl(wanpipe_tdm_api_cmd_t *api_cmd, int cmd);

	int reset_interface_state();

	int start_ring_tone();
	int stop_ring_tone();

	int start_congestion_tone();
	int stop_congestion_tone();

	int start_busy_tone();
	int stop_busy_tone();

	int start_dial_tone();
	int stop_dial_tone();

	int start_ringing_phone();
	int stop_ringing_phone();

	int fxo_go_off_hook();
	int fxo_go_on_hook();

	//more general functions
	int incoming_call(void *aculab_obj, int timeslot, char *incoming_caller_id);
	int incoming_call_idle();

	int outgoing_call(void *aculab_obj, int timeslot);
	int outgoing_call_connected();
	int outgoing_call_idle();

	int tdm_api_event(RX_DATA_STRUCT *pRx);

	int busy_condition();
};

#endif
