//*******************************************************************
//
// Company:		Sangoma Technologies, Inc.
//
// Author:		David Rokhvarg <davidr@sangoma.com>
//
// FileName:	sangoma_interface_manager.h 
//
// Description:	keeps a list of available Sangoma API interfaces
//
// Version:		1.0		05/7/2007	initial version
//
//******************************************************************

#if !defined(AFX_SANGOMA_INTERFACE_MANAGER_H__AED815F2_979C_4B14_B1C4_25C50AC6F3A1__INCLUDED_)
#define AFX_SANGOMA_INTERFACE_MANAGER_H__AED815F2_979C_4B14_B1C4_25C50AC6F3A1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <stdio.h>
#include "sangoma_interface.h"

/////////////////////////
//C++
#include <list>
using namespace std;
//C++
////////////////////////

//debug macro
#define DBG_SANG_IF_MANAGER	if(1)printf
//error reporting macro
#define ERR_SANG_IF_MANAGER	printf
//warning reporting macro
#define WARN_SANG_IF_MANAGER printf

#define IF_MANAGER_FUNC() DBG_SANG_IF_MANAGER("%s::%s():line:%d\n", CLASS_NAME, FUNC_NAME, __LINE__)

//maximum number of cards - just and arbitrary number
#define MAX_NUM_OF_CARDS	5

//maximum number of phones which can be connected to Sangoma (Aanalog) card
#define MAX_NUM_OF_LINES	24

class sangoma_interface_manager  
{
	char	CLASS_NAME[100];

	CRITICAL_SECTION	IfMangerCriticalSection;

	list<sangoma_interface*> if_list;

	//******************************************************************************
	//				start()
	//
	// Description: start ALL interfaces in 'if_list'.
	// Arguments:	none
	// Returns:		0 - success,
	//				non zero - if error occured.
	//******************************************************************************
	int start();

	//******************************************************************************
	//				flush_list_of_interfaces()
	//
	// Description: deallocate list built by build_list_of_interfaces().
	// Arguments:	none
	// Returns:		0 - success,
	//				non zero - if error occured.
	//******************************************************************************
	int flush_list_of_interfaces();

public:

	sangoma_interface_manager();

	virtual ~sangoma_interface_manager();

	//******************************************************************************
	//				build_list_of_interfaces()
	//
	// Description: Scan system an build a list of Sangoma API interfaces.
	// Arguments:	none
	// Returns:		0 - success,
	//				non zero - if error occured.
	//******************************************************************************
	int build_list_of_interfaces(sangoma_callback_functions_t *s_callback_functions);

	//******************************************************************************
	//				find_free_interface()
	//
	// Description: find Sangoma interface not used by a phone call.
	// Arguments:	none
	// Returns:		pointer to a free interface,
	//				NULL if no free interfaces found.
	//******************************************************************************
	sangoma_interface * find_free_interface();
};

#endif // !defined(AFX_SANGOMA_INTERFACE_MANAGER_H__AED815F2_979C_4B14_B1C4_25C50AC6F3A1__INCLUDED_)
